/*
	CBase.cpp
	Classe derivata per interfaccia con Berkeley DB (definisce l'oggetto).
	Luca Piergentili, 04/11/99
	lpiergentili@yahoo.com
	http://www.geocities.com/lpiergentili/
*/
#include "env.h"
#include "pragma.h"
#include <ctype.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "strcpyn.h"
#ifdef _WINDOWS
  #include "window.h"
  #ifdef WIN32_MFC
    #include "CMutexEx.h"
  #endif
#else
  #include "typedef.h"
#endif
#include "CBase.h"
#include "CBerkeleyDB.h"
#include "CDateTime.h"
#include "CTextFile.h"

/*
	CBase()
*/
CBase::CBase(const char* filename,CBASE_TABLE* table,CBASE_INDEX* idx)
{
	// occhio che le funzioni chiamate dal costruttore non possono effettuare il test sulla validita' dell'oggetto
	int nTotFields,nTotSize,nTotIndex,nFieldNum;
	int i,n;

	// valore di ritorno del costruttore
	SetValid(FALSE);

	// puntatore alla struttura della classe base per la tabella
	m_pDatabase = CBerkeleyDB::GetDatabase();

	// imposta la picture di default
	CBase::SetDefaultNumberPicture();
	CBase::SetDefaultCharPicture();
	CBase::SetDefaultPunctPicture();
	CBase::SetDefaultUserPicture();

	// inizializza le strutture interne
	memset(m_pDatabase,'\0',sizeof(DATABASE));
	m_pRecordBuffer = (char*)NULL;

	// ricava il numero di campi e la dim. totale del record
	for(i = 0,nTotFields = 0,nTotSize = 0; table[i].field && *(table[i].field); i++)
	{
		nTotFields++;
		nTotSize += table[i].size;
	}

	// se la tabella prevede indici imposta la dimensione della chiave primaria, incrementando il numero campi e la dim. del record
	int nPrimaryKeySize = (idx==NULL ? 0 : MAX_PRIMARYKEY_SIZE);
	
	if(nPrimaryKeySize > 0)
	{
		nTotFields++;
		nTotSize += nPrimaryKeySize;
	}

	// controlla i valori
	if(nTotFields <= 0 || nTotFields > MAX_FIELDCOUNT)
		return;

	// tot. campi della tabella
	if((m_pDatabase->table.totfield = nTotFields) > 0)
	{
		// nome file per la tabella
		strcpyn(m_pDatabase->table.filename,filename,_MAX_PATH+1);

		// array per la definizione dei campi della tabella
		m_pDatabase->table.row = (ROW*) new char[sizeof(ROW) * nTotFields];
		memset((m_pDatabase->table.row),'\0',sizeof(ROW) * nTotFields);

		// buffer (interno) per il record della tabella
		m_pRecordBuffer = new char[nTotSize + 1];
		memset(m_pRecordBuffer,'\0',nTotSize + 1);

		// imposta la struttura della classe base per la definizione della tabella
		n = 0;
		nFieldNum = 0;

		// se la tabella prevede indici, inserisce il campo per la chiave primaria come primo elemento
		if(nPrimaryKeySize > 0)
		{
			m_pDatabase->table.row[0].num   = 0;
			m_pDatabase->table.row[0].ofs   = n;
			m_pDatabase->table.row[0].name  = "PRIMARY_KEY";
			m_pDatabase->table.row[0].type  = 'C';
			m_pDatabase->table.row[0].size  = nPrimaryKeySize;
			m_pDatabase->table.row[0].dec   = 0;
			m_pDatabase->table.row[0].value = m_pRecordBuffer;
			m_pDatabase->table.row[0].flags = 0;

			n += nPrimaryKeySize;
		}

		// definisce i campi della tabella
		for(i = 0; i < (nPrimaryKeySize > 0 ? nTotFields-1 : nTotFields); i++)
		{
			// numero del campo (salta il campo relativo alla chiave primaria)
			nFieldNum = i + (nPrimaryKeySize > 0 ? 1 : 0);

			// progressivo, offset e nome
			m_pDatabase->table.row[nFieldNum].num   = nFieldNum;
			m_pDatabase->table.row[nFieldNum].ofs   = n;
			m_pDatabase->table.row[nFieldNum].name  = table[i].field;
			
			// tipo
			switch(table[i].type)
			{
				case 'C':
				case 'D':
				case 'L':
				case 'N':
				case 'F':
					break;
				default:
					table[i].type = 'C';
					break;
			}
			m_pDatabase->table.row[nFieldNum].type  = table[i].type;

			// dimensione
			table[i].size = table[i].size > MAX_FIELDSIZE ? MAX_FIELDSIZE : table[i].size;
			m_pDatabase->table.row[nFieldNum].size = table[i].size;

			// decimali (massimo 6 posizioni)
			table[i].dec = table[i].dec > 6 ? 6 : table[i].dec;
			m_pDatabase->table.row[nFieldNum].dec = table[i].dec;

			// offset nel buffer (interno) per il valore del campo
			m_pDatabase->table.row[nFieldNum].value = m_pRecordBuffer + n;
			
			// flags
			m_pDatabase->table.row[nFieldNum].flags = CBASE_FLAG_NONE;

			n += table[i].size;
		}
	}

	// ricava il numero di indici
	if(idx)
	{
		for(i = 0,nTotIndex = 0; idx[i].file && *(idx[i].file); i++)
			nTotIndex++;
	}
	else
		nTotIndex = 0;

	// tot. indici della tabella
	if((m_pDatabase->table.totindex = nTotIndex) > 0)
	{
		// array per la definizione degli indici della tabella
		m_pDatabase->table.index = (INDEX*) new char[sizeof(INDEX) * nTotIndex];
		memset(m_pDatabase->table.index,'\0',sizeof(INDEX) * nTotIndex);

		// definisce gli indici della tabella
		for(i = 0; idx[i].file && *(idx[i].file); i++)
		{
			// nome del file indice
			strcpyn(m_pDatabase->table.index[i].filename,idx[i].file,_MAX_PATH+1);

			// ricava il numero del campo relativo alla chiave
			for(n = 0; n < nTotFields; n++)
			{
				if(strcmp(idx[i].field,table[n].field)==0)
					break;
			}

			if(n==nTotFields)
				n = -1; 

			// nome indice, nome del campo relativo, numero del campo relativo
			if(n >= 0)
			{
				m_pDatabase->table.index[i].name      = idx[i].name;
				m_pDatabase->table.index[i].fieldname = table[n].field;
				m_pDatabase->table.index[i].fieldnum  = n + 1; // il + 1 serve per saltare il campo relativo alla chiave primaria
			}
			else
			{
				m_pDatabase->table.index[i].name      = table[0].field;
				m_pDatabase->table.index[i].fieldname = table[0].field;
				m_pDatabase->table.index[i].fieldnum  = 0;
			}
		}
	}
	
	// valore di ritorno del costruttore
	SetValid(TRUE);
}

/*
	~CBase()
*/
CBase::~CBase()
{
	// chiude la tabella
	CBase::Close();

	// rilascia le pictures
	if(m_pPictureNumber)
	{
		delete [] m_pPictureNumber;
		m_pPictureNumber = NULL;
	}
	if(m_pPictureChar)
	{
		delete [] m_pPictureChar;
		m_pPictureChar = NULL;
	}
	if(m_pPicturePunct)
	{
		delete [] m_pPicturePunct;
		m_pPicturePunct = NULL;
	}
	if(m_pPictureUserDefined)
	{
		delete [] m_pPictureUserDefined;
		m_pPictureUserDefined = NULL;
	}

	// rilascia la struttura per la tabella
	if(m_pDatabase->table.row)
	{
		delete [] m_pDatabase->table.row;
		m_pDatabase->table.row = NULL;
	}

	if(m_pDatabase->table.index)
	{
		delete [] m_pDatabase->table.index;
		m_pDatabase->table.index = NULL;
	}

	// rilascia il record per la tabella
	if(m_pRecordBuffer)
	{
		delete [] m_pRecordBuffer;
		m_pRecordBuffer = NULL;
	}
	
	// imposta lo stato dell'oggetto
	SetValid(FALSE);
}

/*
	Open()
	
	Apre la tabella e gli indici associati, recuperando il primo record.
*/
BOOL CBase::Open(void)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	// apre la tabella e posiziona sul primo record (recuperando i dati)
	if(!CBerkeleyDB::IsOpen())
	{
		if(CBerkeleyDB::Open()==0)
		{
			GetFirst();
			return(TRUE);
		}
	}
	
	return(FALSE);
}

/*
	Zap()

	Elimina tutti i record dalla tabella.
*/
BOOL CBase::Zap(void)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	// azzera la teballa ricreandola
	if(CBerkeleyDB::IsOpen())
		CBerkeleyDB::Close();

	CBerkeleyDB::Create();
	
	m_pDatabase->table.stat.bof = m_pDatabase->table.stat.eof = TRUE;

	return(TRUE);
}

/*
	Close()

	Chiude la tabella e rilascia le risorse associate.
*/
void CBase::Close(void)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	// chiude la tabella
	if(CBerkeleyDB::IsOpen())
		CBerkeleyDB::Close();
}

/*
	GetCurrent()

	Recupera il record corrente della tabella.
	Se non viene specificato altrimenti, utilizza l'indice corrente.
*/
BOOL CBase::GetCurrent(int nIndex/*=-1*/)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	int nCode = DB_OK;

	// imposta l'indice
	if(nIndex >= 0)
	{
		CBerkeleyDB::SetCursor(nIndex);
		
		if((nIndex = CBerkeleyDB::GetCursorIndex()) >= 0)
			nCode = CBerkeleyDB::SetLastError(DB_RETCODE_EINVALIDINDEX);
	}

	// ricava il record corrente
	if(nCode==DB_OK)
	{
		switch((nCode = CBerkeleyDB::GetCurrent()))
		{
			case 0:
			case DB_OK:
				nCode = DB_OK;
				m_pDatabase->table.stat.bof = m_pDatabase->table.stat.eof = FALSE;
				break;

			case DB_RETCODE_BOF:
			case DB_RETCODE_NOTFOUND:
				m_pDatabase->table.stat.bof = TRUE;
				CBerkeleyDB::SetLastError(DB_RETCODE_BOF);
				break;

			default:
				CBerkeleyDB::SetLastError(nCode);
				break;
		}
	}

	return(nCode==DB_OK);
}

/*
	GetFirst()

	Recupera il primo record della tabella.
	Se non viene specificato altrimenti, utilizza l'indice corrente.
*/
BOOL CBase::GetFirst(int nIndex/*=-1*/)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	int nCode = DB_OK;

	// imposta l'indice
	if(nIndex >= 0)
	{
		CBerkeleyDB::SetCursor(nIndex);
		
		if((nIndex = CBerkeleyDB::GetCursorIndex()) >= 0)
			nCode = CBerkeleyDB::SetLastError(DB_RETCODE_EINVALIDINDEX);
	}

	// ricava il primo record
	if(nCode==DB_OK)
	{
		switch((nCode = CBerkeleyDB::GetFirst()))
		{
			case 0:
			case DB_OK:
				nCode = DB_OK;
				m_pDatabase->table.stat.bof = m_pDatabase->table.stat.eof = FALSE;
				break;

			case DB_RETCODE_BOF:
			case DB_RETCODE_NOTFOUND:
				m_pDatabase->table.stat.bof = TRUE;
				CBerkeleyDB::SetLastError(DB_RETCODE_BOF);
				break;

			default:
				CBerkeleyDB::SetLastError(nCode);
				break;
		}
	}

	return(nCode==DB_OK);
}

/*
	GetNext()

	Recupera il record sucessivo della tabella.
	Se non viene specificato altrimenti, utilizza l'indice corrente.
*/
BOOL CBase::GetNext(int nIndex/*=-1*/)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	int nCode = DB_OK;

	// imposta l'indice
	if(nIndex >= 0)
	{
		CBerkeleyDB::SetCursor(nIndex);
		
		if((nIndex = CBerkeleyDB::GetCursorIndex()) >= 0)
			nCode = CBerkeleyDB::SetLastError(DB_RETCODE_EINVALIDINDEX);
	}

	// ricava il record successivo
	if(nCode==DB_OK)
	{
		switch((nCode = CBerkeleyDB::GetNext()))
		{
			case 0:
			case DB_OK:
				nCode = DB_OK;
				m_pDatabase->table.stat.bof = m_pDatabase->table.stat.eof = FALSE;
				break;

			case DB_RETCODE_EOF:
			case DB_RETCODE_NOTFOUND:
				m_pDatabase->table.stat.eof = TRUE;
				CBerkeleyDB::SetLastError(DB_RETCODE_EOF);
				break;

			default:
				CBerkeleyDB::SetLastError(nCode);
				break;
		}
	}

	return(nCode==DB_OK);
}

/*
	GetPrev()

	Recupera il record precedente della tabella.
	Se non viene specificato altrimenti, utilizza l'indice corrente.
*/
BOOL CBase::GetPrev(int nIndex/*=-1*/)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	int nCode = DB_OK;

	// imposta l'indice
	if(nIndex >= 0)
	{
		CBerkeleyDB::SetCursor(nIndex);
		
		if((nIndex = CBerkeleyDB::GetCursorIndex()) >= 0)
			nCode = CBerkeleyDB::SetLastError(DB_RETCODE_EINVALIDINDEX);
	}

	// ricava il record precedente
	if(nCode==DB_OK)
	{
		switch((nCode = CBerkeleyDB::GetPrev()))
		{
			case 0:
			case DB_OK:
				nCode = DB_OK;
				m_pDatabase->table.stat.bof = m_pDatabase->table.stat.eof = FALSE;
				break;

			case DB_RETCODE_BOF:
			case DB_RETCODE_NOTFOUND:
				m_pDatabase->table.stat.bof = TRUE;
				CBerkeleyDB::SetLastError(DB_RETCODE_BOF);
				break;

			default:
				CBerkeleyDB::SetLastError(nCode);
				break;
		}
	}

	return(nCode==DB_OK);
}

/*
	GetLast()

	Recupera l'ultimo record della tabella.
	Se non viene specificato altrimenti, utilizza l'indice corrente.
*/
BOOL CBase::GetLast(int nIndex/*=-1*/)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	int nCode = DB_OK;

	// imposta l'indice
	if(nIndex >= 0)
	{
		CBerkeleyDB::SetCursor(nIndex);
		
		if((nIndex = CBerkeleyDB::GetCursorIndex()) >= 0)
			nCode = CBerkeleyDB::SetLastError(DB_RETCODE_EINVALIDINDEX);
	}

	// ricava l'ultimo record
	if(nCode==DB_OK)
	{
		switch((nCode = CBerkeleyDB::GetLast()))
		{
			case 0:
			case DB_OK:
				nCode = DB_OK;
				m_pDatabase->table.stat.bof = m_pDatabase->table.stat.eof = FALSE;
				break;

			case DB_RETCODE_EOF:
			case DB_RETCODE_NOTFOUND:
				m_pDatabase->table.stat.eof = TRUE;
				CBerkeleyDB::SetLastError(DB_RETCODE_EOF);
				break;

			default:
				CBerkeleyDB::SetLastError(nCode);
				break;
		}
	}

	return(nCode==DB_OK);
}

/*
	GetFieldCount()

	Restituisce il numero di campi della tabella.
*/
int CBase::GetFieldCount(void)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(0);

	// se la tabella prevede indici salta il primo campo (relativo alla chiave primaria)
	return(m_pDatabase->table.totindex > 0 ? m_pDatabase->table.totfield - 1 : m_pDatabase->table.totfield);
}

/*
	GetFieldNames()

	Restituisce i nomi dei campi della tabella.
*/
const char* CBase::GetFieldNames(void)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(NULL);

	char* pFieldName = NULL;
	static int nFieldNum = -1;

	// se la tabella prevede indici salta il primo campo (relativo alla chiave primaria)
	if(nFieldNum < 0)
		nFieldNum = m_pDatabase->table.totindex > 0 ? 1 : 0;

	if(nFieldNum < m_pDatabase->table.totfield)
	{
		pFieldName = m_pDatabase->table.row[nFieldNum].name;
		nFieldNum++;
	}
	else
	{
		pFieldName = NULL;
		nFieldNum = -1;
	}

	return(pFieldName);
}

/*
	GetFieldNumberByName()

	Restituisce il numero del campo (a base 0).
*/
int CBase::GetFieldNumberByName(const char* pFieldName)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(-1);

	int nFieldNum = -1;
	int nTotFields = m_pDatabase->table.totfield;

	// se la tabella prevede indici salta il primo campo (relativo alla chiave primaria)
	if(nTotFields > 0)
	{
		for(nFieldNum = 0; nFieldNum < nTotFields; nFieldNum++)
		{
			if(strcmp(m_pDatabase->table.row[nFieldNum].name,pFieldName)==0)
			{
				nFieldNum = m_pDatabase->table.totindex > 0 ? nFieldNum-1 : nFieldNum;
				break;
			}
		}

	}

	return(nFieldNum);
}

/*
	GetIndexNames()

	Restituisce i nomi degli indici della tabella.
*/
const char* CBase::GetIndexNames(void)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(NULL);

	static int nIndexNum = 0;
	char* pIndexName = NULL;

	if(nIndexNum < m_pDatabase->table.totindex)
	{
		pIndexName = m_pDatabase->table.index[nIndexNum].name;
		nIndexNum++;
	}
	else
	{
		pIndexName = NULL;
		nIndexNum = 0;
	}

	return(pIndexName);
}

/*
	GetIndexNumberByName()

	Restituisce il numero dell'indice (a base 0).
*/
int CBase::GetIndexNumberByName(const char* pIndexName)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(-1);

	int nIndexNum = -1;
	int nTotIndex = m_pDatabase->table.totindex;

	if(nTotIndex > 0)
	{
		for(nIndexNum = 0; nIndexNum < nTotIndex; nIndexNum++)
			if(strcmp(m_pDatabase->table.index[nIndexNum].name,pIndexName)==0)
				return(nIndexNum);
	}

	return(nIndexNum);
}

/*
	SetIndex()

	Imposta l'indice corrente (a base 0), restituendo il valore anteriore.
*/
int CBase::SetIndex(int nIndex)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(-1);

	int nCurrentIndex = CBerkeleyDB::GetCursorIndex();

	CBerkeleyDB::SetCursor(nIndex);

	return(nCurrentIndex);
}

/*
	Seek()

	Cerca la chiave con l'indice specificato.
	Da usare quando la tabella prevede indici.
*/
BOOL CBase::Seek(const int nValue,int nIndex/*=-1*/)
{
	char	szKey[MAX_KEYSIZE+1];
	_snprintf(szKey,sizeof(szKey)-1,"%d",nValue);
	return(Seek(szKey,nIndex));
}

/*
	Seek()

	Cerca la chiave con l'indice specificato.
	Da usare quando la tabella prevede indici.
*/
BOOL CBase::Seek(const char* pValue,int nIndex/*=-1*/)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	int	nCode = DB_OK;
	int	nKeySize;
	char	szKey[MAX_KEYSIZE+1];
	int	nFieldNum,nFieldSize;
	char	cFieldType;

	// imposta l'indice corrente
	if(nIndex >= 0)
		CBerkeleyDB::SetCursor(nIndex);

	// se la tabella prevede indici
	if((nIndex = CBerkeleyDB::GetCursorIndex()) >= 0)
	{
		// imposta la chiave, allineando a seconda del tipo del campo (stringhe a sx, numeri a dx)
		memset(szKey,' ',sizeof(szKey));
		nKeySize = strlen(pValue);
		nKeySize = nKeySize >= sizeof(szKey) ? sizeof(szKey) : nKeySize;

		nFieldNum  = m_pDatabase->table.index[nIndex].fieldnum;
		nFieldSize = m_pDatabase->table.row[nFieldNum].size;
		cFieldType = m_pDatabase->table.row[nFieldNum].type;

		if(cFieldType=='N' || cFieldType=='F')
			memcpy(szKey + (nFieldSize-nKeySize),pValue,nKeySize);
		else
			memcpy(szKey,pValue,nKeySize);

		// imposta il campo (chiave) per la ricerca
		CBerkeleyDB::PutKey(szKey,nIndex);

		// ricava il record relativo alla chiave
		nCode = CBerkeleyDB::GetCurrent();
	}
	else
		nCode = DB_RETCODE_EINVALIDINDEX;

	// esito ricerca
	switch(nCode)
	{
		case 0:
		case DB_OK:
			nCode = DB_OK;
			break;

		case DB_RETCODE_EOF:
		case DB_RETCODE_NOTFOUND:
			m_pDatabase->table.stat.eof = TRUE;
			CBerkeleyDB::SetLastError(DB_RETCODE_NOTFOUND);
			break;

		default:
			CBerkeleyDB::SetLastError(nCode);
			break;
	}

	return(nCode==DB_OK);
}

/*
	Find()

	Cerca il record relativo alla chiave.
	Da usare quando la tabella non prevede indici (notare che in tal caso viene considerato come chiave
	l'intero record).
*/
BOOL CBase::Find(const char* pValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	int	nCode = DB_OK;
	int	nKeySize;
	char	szKey[MAX_KEYSIZE+1];
	int	nFieldNum,nFieldSize;
	char	cFieldType;

	// imposta la chiave, allineando a seconda del tipo del campo (stringhe a sx, numeri a dx)
	memset(szKey,' ',sizeof(szKey));
	nKeySize = strlen(pValue);
	nKeySize = nKeySize >= sizeof(szKey) ? sizeof(szKey) : nKeySize;

	nFieldNum  = 0;
	nFieldSize = m_pDatabase->table.row[nFieldNum].size;
	cFieldType = m_pDatabase->table.row[nFieldNum].type;

	if(cFieldType=='N' || cFieldType=='F')
		memcpy(szKey + (nFieldSize-nKeySize),pValue,nKeySize);
	else
		memcpy(szKey,pValue,nKeySize);

	// imposta il campo (chiave) per la ricerca
	CBerkeleyDB::PutKey(szKey,0);

	// ricava il record relativo alla chiave
	nCode = CBerkeleyDB::GetCurrent();

	// esito ricerca
	switch(nCode)
	{
		case 0:
		case DB_OK:
			nCode = DB_OK;
			break;

		case DB_RETCODE_EOF:
		case DB_RETCODE_NOTFOUND:
			m_pDatabase->table.stat.eof = TRUE;
			CBerkeleyDB::SetLastError(DB_RETCODE_NOTFOUND);
			break;

		default:
			CBerkeleyDB::SetLastError(nCode);
			break;
	}

	return(nCode==DB_OK);
}

/*
	PutField()

	Imposta il campo interno (relativo al campo della tabella) con quanto passato in input.
	La chiamata modifica il campo interno, NON il campo della tabella, da aggiornare con i metodi
	relativi (insert, update, etc.).
*/
void CBase::PutField(int nFieldNum,const char* pValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	char szField[MAX_KEYSIZE+1];
	
	// se la tabella prevede indici salta il primo campo (relativo alla chiave primaria)
	nFieldNum = m_pDatabase->table.totindex > 0 ? nFieldNum + 1 : nFieldNum;

	if(nFieldNum >= 0 && nFieldNum < m_pDatabase->table.totfield)
	{
		memset(szField,'\0',sizeof(szField));
		strncpy(szField,pValue,sizeof(szField)-1);

		// formatta il contenuto del buffer prima di passarlo nel campo
		if(m_pDatabase->table.row[nFieldNum].flags!=CBASE_FLAG_NONE)
			CBase::SetFieldFormat(szField,strlen(szField),m_pDatabase->table.row[nFieldNum].flags);

		// passa il contenuto del buffer nel campo interno (relativo al campo della tabella)
		CBerkeleyDB::PutField(nFieldNum,szField);
	}
}

void CBase::PutField(int nFieldNum,BOOLEAN bValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	// da boolean a carattere
	char szValue[5];
	_snprintf(szValue,sizeof(szValue)-1,"%d",bValue ? 1 : 0);
	
	PutField(nFieldNum,szValue);
}

void CBase::PutField(int nFieldNum,CDateTime* pDateTime)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	// da data a carattere
	char szValue[16];
	if(pDateTime->GetYear()==0 && pDateTime->GetMonth()==0 && pDateTime->GetDay()==0)
		strcpy(szValue,"        ");
	else
		_snprintf(szValue,sizeof(szValue)-1,"%.4d%.2d%.2d",pDateTime->GetYear(),pDateTime->GetMonth(),pDateTime->GetDay());

	PutField(nFieldNum,szValue);
}

void CBase::PutField(int nFieldNum,CDateTime& pDateTime)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	// da ora a carattere
	char szValue[16];
	_snprintf(szValue,sizeof(szValue)-1,"%.4d%.2d%.2d",pDateTime.GetYear(),pDateTime.GetMonth(),pDateTime.GetDay());
	
	PutField(nFieldNum,szValue);
}

void CBase::PutField(int nFieldNum,int nValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	// da numero a intero
	NUMBER Ni = {0};
	Ni.integer = nValue;
	
	PutFieldNumber(nFieldNum,Ni,integer_type);
}

void  CBase::PutField(int nFieldNum,long nValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	// da numero a long
	NUMBER Nl = {0};
	Nl.longint = nValue;

	PutFieldNumber(nFieldNum,Nl,longint_type);
}

void CBase::PutField(int nFieldNum,unsigned long nValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	// da numero a unsigned long
	NUMBER Nu = {0};
	Nu.ulongint = nValue;
	
	PutFieldNumber(nFieldNum,Nu,ulongint_type);
}

void CBase::PutField(int nFieldNum,float nValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	// da numero a float
	NUMBER Nf = {0};
	Nf.floatnum = nValue;
	
	PutFieldNumber(nFieldNum,Nf,floatnum_type);
}

void CBase::PutFieldNumber(int nFieldNum,NUMBER value,NUMTYPE type)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	char cFieldType;
	int  nFieldNumber,nFieldLen,nFieldSize,nFieldDec;
	char szField[MAX_FIELDSIZE+1];
	char szBuffer[MAX_FIELDSIZE+1];
	char szFormat[10];

	nFieldNumber = nFieldNum;

	// se la tabella prevede indici salta il primo campo (relativo alla chiave primaria)
	nFieldNum = m_pDatabase->table.totindex > 0 ? nFieldNum + 1 : nFieldNum;

	if(nFieldNum >= 0 && nFieldNum < m_pDatabase->table.totfield)
	{
		memset(szField,' ',sizeof(szField));

		// tipo, dimensione e decimali del campo
		cFieldType = m_pDatabase->table.row[nFieldNum].type;
		nFieldSize = m_pDatabase->table.row[nFieldNum].size;
		nFieldDec  = m_pDatabase->table.row[nFieldNum].dec;
		nFieldDec  = nFieldDec > 6 ? 6 : nFieldDec;
		
		switch(type)
		{
			case integer_type:
				_snprintf(szBuffer,sizeof(szBuffer)-1,"%d",value.integer);
				break;
			case longint_type:
				_snprintf(szBuffer,sizeof(szBuffer)-1,"%ld",value.longint);
				break;
			case ulongint_type:
				_snprintf(szBuffer,sizeof(szBuffer)-1,"%lu",value.ulongint);
				break;
			case floatnum_type:
				_snprintf(szFormat,sizeof(szFormat)-1,"%c.%d%c",'%',nFieldDec,'f');
				_snprintf(szBuffer,sizeof(szBuffer)-1,szFormat,value.floatnum);
				break;
		}

		nFieldLen = strlen(szBuffer);
		nFieldLen = nFieldLen > nFieldSize ? nFieldSize : nFieldLen;
		
		// allineamento (stringhe a sx, numeri a dx)
		if(cFieldType=='N' || cFieldType=='F')
			memcpy(szField + (nFieldSize-nFieldLen),szBuffer,nFieldLen);
		else
			memcpy(szField,szBuffer,nFieldLen);
		
		szField[nFieldSize] = '\0';
		
		CBase::PutField(nFieldNumber,(const char*)szField);
	}
}

/*
	GetField()

	Ricava il valore del campo interno (relativo al campo della tabella).
	Il valore restituito e' quello relativo al record corrente.
*/
const char* CBase::GetField(int nFieldNum)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(NULL);

	// se la tabella prevede indici salta il primo campo (relativo alla chiave primaria)
	nFieldNum = m_pDatabase->table.totindex > 0 ? nFieldNum + 1 : nFieldNum;

	// restituisce il puntatore a un buffer interno, sovrascritto dalle chiamate successive
	return(CBerkeleyDB::GetField(nFieldNum));
}

char* CBase::GetField(int nFieldNum,char* pBuffer,int nSize)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(NULL);

	// se la tabella prevede indici salta il primo campo (relativo alla chiave primaria)
	nFieldNum = m_pDatabase->table.totindex > 0 ? nFieldNum + 1 : nFieldNum;

	// size deve corrispondere alla dim. del buffer - 1, ossia il buffer deve essere di size + 1 caratteri
	memset(pBuffer,'\0',nSize);
	memcpy(pBuffer,CBerkeleyDB::GetField(nFieldNum),nSize);
	pBuffer[nSize] = '\0';

	return(pBuffer);
}

BOOLEAN CBase::GetField(int nFieldNum,BOOLEAN& bValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	// da carattere a boolean
	char* pValue = (char*)CBase::GetField(nFieldNum);

	if(pValue[0]=='0')
		bValue = FALSE;
	else if(pValue[0]=='1')
		bValue = TRUE;

	return(bValue);
}

const char* CBase::GetField(int nFieldNum,CDateTime* pDateTime)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(NULL);

	// da carattere a data
	static char date[64];
	char year[5]  = {0};
	char month[3] = {0};
	char day[3]   = {0};

	memset(date,'\0',sizeof(date));
	strncpy(date,CBase::GetField(nFieldNum),sizeof(date)-1);
	
	memcpy(year,date,4);
	memcpy(month,date+4,2);
	memcpy(day,date+6,2);

	pDateTime->SetDay(atoi(day));
	pDateTime->SetMonth(atoi(month));
	pDateTime->SetYear(atoi(year));

	memset(date,'\0',sizeof(date));
	
	if(pDateTime->GetYear()==0 && pDateTime->GetMonth()==0 && pDateTime->GetDay()==0)
		strcpy(date,"        ");
	else
		strncpy(date,pDateTime->GetFormattedDate(FALSE),sizeof(date)-1);

	return(date);
}

const char* CBase::GetField(int nFieldNum,CDateTime& pDateTime)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(NULL);

	// da carattere a data
	static char date[64];
	char year[5]  = {0};
	char month[3] = {0};
	char day[3]   = {0};

	memset(date,'\0',sizeof(date));
	strncpy(date,CBase::GetField(nFieldNum),sizeof(date)-1);
	memcpy(year,date,4);
	memcpy(month,date+4,2);
	memcpy(day,date+6,2);

	pDateTime.SetDay(atoi(day));
	pDateTime.SetMonth(atoi(month));
	pDateTime.SetYear(atoi(year));

	memset(date,'\0',sizeof(date));
	strncpy(date,pDateTime.GetFormattedDate(FALSE),sizeof(date)-1);

	return(date);
}

int CBase::GetField(int nFieldNum,int& nValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(-1);

	// da numero a intero
	NUMBER Ni = {0};
	GetFieldNumber(nFieldNum,Ni,integer_type);
	nValue = Ni.integer;

	return(nValue);
}

long CBase::GetField(int nFieldNum,long& nValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(-1L);

	// da numero a long
	NUMBER Nl = {0};
	GetFieldNumber(nFieldNum,Nl,longint_type);
	nValue = Nl.longint;

	return(nValue);
}

unsigned long CBase::GetField(int nFieldNum,unsigned long& nValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return((unsigned long)-1L);

	// da numero a unsigned long
	NUMBER Nu = {0};
	GetFieldNumber(nFieldNum,Nu,ulongint_type);
	nValue = Nu.ulongint;

	return(nValue);
}

float CBase::GetField(int nFieldNum,float& nValue)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return((float)-1);

	// da numero a float
	NUMBER Nf = {0};
	GetFieldNumber(nFieldNum,Nf,floatnum_type);
	nValue = Nf.floatnum;

	return(nValue);
}

void CBase::GetFieldNumber(int nFieldNum,NUMBER& value,NUMTYPE type)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	char szField[MAX_FIELDSIZE + 1];

	// se la tabella prevede indici salta il primo campo (relativo alla chiave primaria)
	nFieldNum = m_pDatabase->table.totindex > 0 ? nFieldNum + 1 : nFieldNum;

	if(nFieldNum >= 0 && nFieldNum < m_pDatabase->table.totfield)
	{
		memset(szField,'\0',sizeof(szField));
		strncpy(szField,CBerkeleyDB::GetField(nFieldNum),sizeof(szField)-1);

		switch(type)
		{
			case integer_type:
				value.integer = atoi(szField);
				break;
			case longint_type:
				value.longint = atol(szField);
				break;
			case ulongint_type:
				value.ulongint = atol(szField);
				break;
			case floatnum_type:
				value.floatnum = (float)atof(szField);
				break;
		}
	}
}

/*
	SetNumberPicture()

	Imposta la picture numerica.
*/
BOOL CBase::SetNumberPicture(const char* pPicture)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	if(m_pPictureNumber)
	{
		memset(m_pPictureNumber,'\0',CBASE_MAX_NUMBER_PICTURE + 1);
		strncpy(m_pPictureNumber,pPicture,CBASE_MAX_NUMBER_PICTURE);
		return(TRUE);
	}
	else
		return(FALSE);
}

/*
	SetCharPicture()

	Imposta la picture carattere.
*/
BOOL CBase::SetCharPicture(const char* pPicture)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	if(m_pPictureChar)
	{
		memset(m_pPictureChar,'\0',CBASE_MAX_CHAR_PICTURE + 1);
		strncpy(m_pPictureChar,pPicture,CBASE_MAX_CHAR_PICTURE);
		return(TRUE);
	}
	else
		return(FALSE);
}

/*
	SetPunctPicture()

	Imposta la picture per la punteggiatura.
*/
BOOL CBase::SetPunctPicture(const char* pPicture)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	if(m_pPicturePunct)
	{
		memset(m_pPicturePunct,'\0',CBASE_MAX_PUNCT_PICTURE + 1);
		strncpy(m_pPicturePunct,pPicture,CBASE_MAX_PUNCT_PICTURE);
		return(TRUE);
	}
	else
		return(FALSE);
}

/*
	SetUserPicture()

	Imposta la picture definita dall'utente.
*/
BOOL CBase::SetUserPicture(const char* pPicture)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(FALSE);

	if(m_pPictureUserDefined)
	{
		memset(m_pPictureUserDefined,'\0',CBASE_MAX_USER_PICTURE + 1);
		strncpy(m_pPictureUserDefined,pPicture,CBASE_MAX_USER_PICTURE);
		return(TRUE);
	}
	else
		return(FALSE);
}

/*
	SetFieldFlags()

	Imposta i flags per il campo.
*/
void CBase::SetFieldFlags(int nFieldNum,unsigned long nFlags)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	// se la tabella prevede indici salta il primo campo (relativo alla chiave primaria)
	nFieldNum = m_pDatabase->table.totindex > 0 ? nFieldNum + 1 : nFieldNum;
	
	if(nFieldNum >= 0 && nFieldNum < m_pDatabase->table.totfield)
		m_pDatabase->table.row[nFieldNum].flags = nFlags;
}

/*
	SetDefaultNumberPicture()

	Imposta la picture numerica di default.
*/
void CBase::SetDefaultNumberPicture(void)
{
	m_pPictureNumber = new char[CBASE_MAX_NUMBER_PICTURE + 1];
	
	if(m_pPictureNumber)
	{
		memset(m_pPictureNumber,'\0',CBASE_MAX_NUMBER_PICTURE + 1);
//$		strncpy(m_pPictureNumber,"0123456789",CBASE_MAX_NUMBER_PICTURE);
	}
}

/*
	SetDefaultCharPicture()

	Imposta la picture carattere di default.
*/
void CBase::SetDefaultCharPicture(void)
{
	m_pPictureChar = new char[CBASE_MAX_CHAR_PICTURE + 1];
	
	if(m_pPictureChar)
	{
		memset(m_pPictureChar,'\0',CBASE_MAX_CHAR_PICTURE + 1);
//$		strncpy(m_pPictureChar," abcdefghilmnopqrstuvzABCDEFGHILMNOPQRSTUVZ",CBASE_MAX_CHAR_PICTURE);
	}
}

/*
	SetDefaultPunctPicture()

	Imposta la picture per la punteggiatura di default.
*/
void CBase::SetDefaultPunctPicture(void)
{
	m_pPicturePunct = new char[CBASE_MAX_PUNCT_PICTURE + 1];
	
	if(m_pPicturePunct)
	{
		memset(m_pPicturePunct,'\0',CBASE_MAX_PUNCT_PICTURE + 1);
//$		strncpy(m_pPicturePunct,".,:;",CBASE_MAX_PUNCT_PICTURE);
	}
}

/*
	SetDefaultUserPicture()

	Imposta la picture utente di default.
*/
void CBase::SetDefaultUserPicture(void)
{
	m_pPictureUserDefined = new char[CBASE_MAX_USER_PICTURE + 1];
	
	if(m_pPictureUserDefined)
		memset(m_pPictureUserDefined,'\0',CBASE_MAX_USER_PICTURE + 1);
}

/*
	SetFieldFormat()

	Formatta il contenuto del campo a seconda dei flags.
*/
void CBase::SetFieldFormat(char* pField,int nSize,unsigned long nFlags)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	if(nFlags & CBASE_FLAG_TOUPPER)
		strupr(pField);
	if(nFlags & CBASE_FLAG_TOLOWER)
		strlwr(pField);
	if((nFlags & CBASE_FLAG_NUMBER && *m_pPictureNumber) || (nFlags & CBASE_FLAG_CHAR && *m_pPictureChar) || (nFlags & CBASE_FLAG_PUNCT && *m_pPicturePunct) || (nFlags & CBASE_FLAG_USERDEFINED && *m_pPictureUserDefined))
		CBase::SetFieldFormatByPicture(pField,nSize,nFlags);
}

/*
	SetFieldFormatByPicture()

	Formatta il contenuto del campo secondo la picture.
*/
void CBase::SetFieldFormatByPicture(char* pField,int nSize,unsigned long nFlags)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return;

	char szBuffer[MAX_FIELDSIZE+1];
	register char* p = szBuffer;
	register int i;

	memset(szBuffer,' ',sizeof(szBuffer));

	for(i = 0; pField[i] && i < nSize && i < sizeof(szBuffer); i++)
	{
		if(nFlags & CBASE_FLAG_NUMBER)
			if(strchr(m_pPictureNumber,pField[i]))
			{
				*p++ = pField[i];
				continue;
			}
		
		if(nFlags & CBASE_FLAG_CHAR)
			if(strchr(m_pPictureChar,pField[i]))
			{
				*p++ = pField[i];
				continue;
			}
		
		if(nFlags & CBASE_FLAG_PUNCT)
			if(strchr(m_pPictureChar,pField[i]))
			{
				*p++ = pField[i];
				continue;
			}
		
		if(nFlags & CBASE_FLAG_USERDEFINED)
			if(strchr(m_pPictureUserDefined,pField[i]))
			{
				*p++ = pField[i];
				continue;
			}
	}

	memset(pField,' ',nSize);

	if((i = p-szBuffer) > 0)
	{
		// allineamento (stringhe a sx, numeri a dx)
		if(nFlags==CBASE_FLAG_NUMBER) //$ == e non &
			memcpy(pField + (nSize-i),szBuffer,i);
		else
			memcpy(pField,szBuffer,i);
	}
}

/*
	Load()

	Carica la tabella con quanto contenuto nel file SDF.
	Assume che il file SDF contenga solo dati, senza il valore della chiave primaria
	(generato da Dump() se viene specificato TRUE come secondo parametro).
*/
int CBase::Load(const char* pFileName)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(DB_RETCODE_EINVALIDHANDLE);

	CTextFile textfile;
	char szField[MAX_KEYSIZE + 1];
	int i,nPrimaryKeySize,nRecordSize,nFieldLen;

	// la tabella non deve essere gia' aperta
	if(CBerkeleyDB::IsOpen())
		return(CBerkeleyDB::SetLastError(DB_RETCODE_EALREDYOPEN));

	// apre la tabella in proprio
	if(CBerkeleyDB::Open()!=0)
		return(CBerkeleyDB::SetLastError(DB_RETCODE_EOPENTABLE));

	// dimensione della chiave primaria
	if(m_pDatabase->table.totindex > 0)
		nPrimaryKeySize = CBerkeleyDB::GetPrimaryKeySize();
	else
		nPrimaryKeySize = 0;

	// dimensione del record (salta la chiave primaria)
	nRecordSize = 0;
	for(i = m_pDatabase->table.totindex > 0 ? 1 : 0; i < m_pDatabase->table.totfield; i++)
		nRecordSize += m_pDatabase->table.row[i].size;
	
	if(nRecordSize <= 0)
	{
		CBerkeleyDB::Close();
		return(CBerkeleyDB::SetLastError(DB_RETCODE_EINVALIDFIELDSIZE));
	}

	// buffer per la lettura del record dal file sdf (dim. record + crlf + null)
	char* pBuffer = new char[nRecordSize + 2 + 1];
	if(!pBuffer)
	{
		CBerkeleyDB::Close();
		return(CBerkeleyDB::SetLastError(DB_RETCODE_EALLOCFAILURE));
	}

	// chiude la tabella, la elimina (insieme agli indici relativi) e la ricrea per il caricamento
	CBerkeleyDB::Close();

	remove(m_pDatabase->table.filename);

	if(m_pDatabase->table.totindex > 0)
		for(i = 0; i < m_pDatabase->table.totindex; i++)
			remove(m_pDatabase->table.index[i].filename);

	if(CBerkeleyDB::Open()!=0)
		return(CBerkeleyDB::SetLastError(DB_RETCODE_EOPENTABLE));

	// carica la tabella con il contenuto del file SDF
	if(textfile.Open(pFileName))
	{
		// azzera il buffer interno
		memset(m_pRecordBuffer,' ',nRecordSize + nPrimaryKeySize + 1);

		// legge una linea dal file
		while(textfile.ReadLine(pBuffer,nRecordSize + 2 + 1)!=FILE_EOF)
		{
			// passa la linea nel buffer interno (salta la chiave primaria)
			memcpy(m_pRecordBuffer + nPrimaryKeySize,pBuffer,nRecordSize);

			// filtra il contenuto del campo secondo la picture corrente
			for(i = 0; i < m_pDatabase->table.totfield; i++)
			{
				if(m_pDatabase->table.row[i].flags!=CBASE_FLAG_NONE)
				{
					memset(szField,'\0',sizeof(szField));
					memcpy(szField,m_pDatabase->table.row[i].value,m_pDatabase->table.row[i].size);
					nFieldLen = strlen(szField);
					
					CBase::SetFieldFormat(szField,nFieldLen,m_pDatabase->table.row[i].flags);
					
					memset(m_pDatabase->table.row[i].value,' ',m_pDatabase->table.row[i].size);
					memcpy(m_pDatabase->table.row[i].value,szField,nFieldLen);
				}
			}

			// inserisce il record nella tabella
			CBase::Insert();
		}

		textfile.Close();
	}
	else
	{
		CBerkeleyDB::Close();
		return(CBerkeleyDB::SetLastError(DB_RETCODE_EOPENFAILURE));
	}

	delete [] pBuffer;
	
	CBerkeleyDB::Close();

	return(DB_OK);
}

/*
	Dump()

	Scarica nel file SDF quanto contenuto nella tabella.
	Il secondo parametro serve per stampare il valore della chiave primaria.
	Notare che Load() assume che il file SDF contenga solo dati e non anche
	il valore della chiave primaria.
*/
int CBase::Dump(const char* pFileName,BOOL bDumpPrimaryKey)
{
	// controlla il codice di ritorno del costruttore
	if(!IsValid())
		return(DB_RETCODE_EINVALIDHANDLE);

	CTextFile textfile;
	int nRecordSize;
	int nPrimaryKeySize;
	int i;

	// la tabella deve essere gia' aperta
	if(!CBerkeleyDB::IsOpen())
		return(CBerkeleyDB::SetLastError(DB_RETCODE_ETABLENOTOPEN));

	// dimensione della chiave primaria
	if(m_pDatabase->table.totindex > 0)
		nPrimaryKeySize = CBerkeleyDB::GetPrimaryKeySize();
	else
		nPrimaryKeySize = 0;

	// dimensione del record (non include la dimensione della chiave primaria)
	nRecordSize = 0;
	for(i = m_pDatabase->table.totindex > 0 ? 1 : 0; i < m_pDatabase->table.totfield; i++)
		nRecordSize += m_pDatabase->table.row[i].size;
	
	if(nRecordSize <= 0)
		return(CBerkeleyDB::SetLastError(DB_RETCODE_EINVALIDFIELDSIZE));

	remove(pFileName);

	// scarica il contenuto della tabella nel file SDF
	if(textfile.Create(pFileName))
	{
		memset(m_pRecordBuffer,'\0',nRecordSize + nPrimaryKeySize + 1);

		// include o salta la chiave primaria
		char* p = bDumpPrimaryKey ? m_pRecordBuffer : m_pRecordBuffer + nPrimaryKeySize;
		int   n = bDumpPrimaryKey ? nRecordSize + nPrimaryKeySize : nRecordSize;

		if(CBase::GoTop())
		{
			while(!CBase::Eof())
			{
				textfile.WriteLine(p,n);
				CBase::GetNext();
			}
		}

		textfile.Close();
	}
	else
		return(CBerkeleyDB::SetLastError(DB_RETCODE_EOPENFAILURE));

	return(DB_OK);
}
