/*
	CFindFile.cpp
	Classe base per la ricerca files (SDK).
	Luca Piergentili, 14/02/00
	lpiergentili@yahoo.com
	http://www.geocities.com/lpiergentili/
*/
#include "env.h"
#include "pragma.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "strcpyn.h"
#include "window.h"
#include "win32api.h"
#include "CNodeList.h"
#include "CFindFile.h"

/*
	CFindFile()
*/
CFindFile::CFindFile()
{
	m_nCount = 0;
	m_nInternalCount = -1;
	m_bFindFirstCall = TRUE;
	m_bFindExFirstCall = TRUE;
	memset(&m_FindFile,'\0',sizeof(FINDFILE));
	m_FindFile.handle = INVALID_HANDLE_VALUE;
	memset(&m_FindData,'\0',sizeof(WIN32_FIND_DATA));
}

/*
	~CFindFile()
*/
CFindFile::~CFindFile()
{
	Close();
}

/*
	Exist()

	Controlla se il file esiste.
*/
BOOL CFindFile::Exist(LPCSTR lpcszFileName)
{    
	BOOL bExist = FALSE;
	HANDLE hHandle;

	if((hHandle = ::CreateFile(lpcszFileName,0,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL))!=INVALID_HANDLE_VALUE)
		bExist = ::CloseHandle(hHandle);

	return(bExist);
}

/*
	ExistEx()
	
	Controlla se lo skeleton esiste.
	Passare il pathname completo + lo skeleton ("c:\*.exe" or "c:\file.exe").
*/
BOOL CFindFile::ExistEx(LPCSTR lpcszSkel)
{    
	BOOL bExist = FALSE;
    	
	if(CFindFile::First(lpcszSkel,_A_NORMAL|_A_RDONLY|_A_HIDDEN|_A_SYSTEM))
		bExist = TRUE;

	CFindFile::Close();

	return(bExist);
}

/*
	Find()
	
	Ricerca lo skeleton specificato.
	Passare il pathname completo + lo skeleton ("c:\*.exe" or "c:\file.exe").
	Chiamare in un ciclo fino a che non restituisce NULL.
*/
LPCSTR CFindFile::Find(LPCSTR lpcszSkel/* = "\\*.*" */)
{    
	char* p;
	LPSTR lpFile = NULL;
	static char szPathName[_MAX_PATH+1];
	static char szFileName[_MAX_PATH+1];

	// ricava il pathname
	memset(szPathName,'\0',sizeof(szPathName));
	strcpyn(szFileName,lpcszSkel,sizeof(szFileName));
	strrev(szFileName);
	if((p = strchr(szFileName,'\\'))==(char*)NULL)
		p = strchr(szFileName,':');
	if(p)
	{
		strcpyn(szPathName,p,sizeof(szPathName));
		strrev(szPathName);
	}
	memset(szFileName,'\0',sizeof(szFileName));
    	
	// cerca la prima istanza dello skeleton
	if(m_bFindFirstCall)
	{
		m_bFindFirstCall = FALSE;
		m_nCount = 0;
	     
		// i files trovati corrispondono allo skeleton
		if(CFindFile::First(lpcszSkel,_A_NORMAL|_A_RDONLY|_A_HIDDEN|_A_SYSTEM))
		{
			if(m_FindFile.attrib & _A_SUBDIR)
				goto next;

			strcpyn(szFileName,m_FindFile.name,sizeof(szFileName));
			strcat(szPathName,szFileName);
			lpFile = szPathName;
			m_nCount++;

			// se e' stato specificato un pathname completo e non uno skeleton ("c:\\file.ext"), termina la ricerca
			if(strchr(lpcszSkel,'?')==(char*)NULL && strchr(lpcszSkel,'*')==(char*)NULL)
			{
				CFindFile::Close();
				m_bFindFirstCall = TRUE;
			}
		}
		else
		{
			CFindFile::Close();
			m_bFindFirstCall = TRUE;
		}
    }
    else // cerca le istanze successive dello skeleton
    {
next:
		// trovato il file seguente
		if(CFindFile::Next())
		{
			// esclude "." e ".."
			if(m_FindFile.attrib & _A_SUBDIR)
				goto next;

			strcpyn(szFileName,m_FindFile.name,sizeof(szFileName));
			strcat(szPathName,szFileName);
			lpFile = szPathName;
			m_nCount++;
		}
		else
		{
			CFindFile::Close();
			m_bFindFirstCall = TRUE;
		}
	}

	return(lpFile);
}

/*
	FindEx()
	
	Cerca lo skeleton specificato a partire dalla directory iniziale (terminare con '\').
	Chiamare in un ciclo fino a che non restituisce NULL.
*/
LPCSTR CFindFile::FindEx(LPCSTR lpcszStartDir/* = "\\" */,LPCSTR lpcszSkel/* = "*.*" */,BOOL bRecursive/* = TRUE */,UINT uAttribute/* = _A_ALLFILES */)
{
	LPSTR lpFile = NULL;

	// creates the list
	if(m_bFindExFirstCall)
	{
		m_nCount = 0;
		m_FileList.DeleteAll();

		// cerca lo skeleton		
		HANDLE hHandle;
		if((hHandle = CFindFile::Search(lpcszStartDir,lpcszSkel,bRecursive,uAttribute))==INVALID_HANDLE_VALUE)
			return(NULL);
		
		m_bFindExFirstCall = FALSE;
		CFindFile::Close(hHandle);

		m_nCount = m_FileList.Count();
	}

	// restituisce quanto presente nella lista
	if(!m_bFindExFirstCall)
	{
		if(m_FileList.Count() > 0)
		{
			if(m_nInternalCount==-1)
				m_nInternalCount = 0;

			FINDFILE* f;
			if((f = (FINDFILE*)m_FileList.GetAt(m_nInternalCount))!=(FINDFILE*)NULL)
			{
				lpFile = f->name;
				m_nInternalCount++;
			}
			else
			{
				lpFile = NULL;
				m_FileList.DeleteAll();
				m_bFindExFirstCall = TRUE;
				m_nInternalCount = -1;
			}
		}
		else
		{
			m_bFindExFirstCall = TRUE;
			m_nInternalCount = -1;
		}
	}

	return(lpFile);
}

/*
	FindFile()
	
	Cerca lo skeleton specificato a partire dalla directory iniziale (terminare con '\').
	I files trovati vengono inseriti nella lista interna e devono essere recuperati con la GetAt().
*/
UINT CFindFile::FindFile(LPCSTR lpcszStartDir/* = "\\" */,LPCSTR lpcszSkel/* = "*.*" */,BOOL bRecursive/* = TRUE */,UINT uAttribute/* = _A_ALLFILES */)
{
	m_FileList.DeleteAll();
		
	// cerca lo skeleton
	HANDLE hHandle;
	if((hHandle = CFindFile::Search(lpcszStartDir,lpcszSkel,bRecursive,uAttribute))!=INVALID_HANDLE_VALUE)
		CFindFile::Close(hHandle);

	m_nCount = m_FileList.Count();

	return(m_nCount);
}

/*
	GetAt()
	
	Restituisce quanto presente nella lista interna.
	Chiamare in un ciclo che vada da 0 a Count().
*/
LPCSTR CFindFile::GetAt(int nIndex)
{
	LPSTR lpFile = NULL;
	
	if(m_FileList.Count() > 0)
	{
		if(nIndex >= 0 && nIndex < m_FileList.Count())
		{
			FINDFILE* f;
			if((f = (FINDFILE*)m_FileList.GetAt(nIndex))!=(FINDFILE*)NULL)
				lpFile = f->name;
		}
	}

	return(lpFile);
}

/*
	Search()
	
	Cerca lo skeleton specificato a partire dalla directory iniziale (terminare con '\').
	I files trovati vengono inseriti nella lista interna.
*/
HANDLE CFindFile::Search(LPCSTR lpcszStartDir,LPCSTR lpcszSkel,BOOL bRecursive,UINT uAttribute)
{
	FINDFILE f;
	char szPathName[_MAX_PATH+_MAX_FNAME+1];

	if(lpcszStartDir[strlen(lpcszStartDir)-1]!='\\')
		return(INVALID_HANDLE_VALUE);

	if((strlen(lpcszStartDir) + strlen(lpcszSkel)) < sizeof(szPathName))
		_snprintf(szPathName,sizeof(szPathName)-1,"%s%s",lpcszStartDir,lpcszSkel);
	else
		return(INVALID_HANDLE_VALUE);

	// cerca la prima istanza dello skeleton
	if(CFindFile::First(szPathName,uAttribute,&f))
	{
		do
		{
			// considera solo i nomi file
			if(f.name[0]!='.')
			{
				// pathname completo
				if((strlen(lpcszStartDir) + strlen(f.name)) < sizeof(szPathName))
					_snprintf(szPathName,sizeof(szPathName)-1,"%s%s",lpcszStartDir,f.name);
				else
					return(INVALID_HANDLE_VALUE);

				// inserisce il nome file nella lista
				FINDFILE* ff = new FINDFILE;
				if(ff)
				{
					ff->attrib  = f.attrib;
					ff->wr_time = f.wr_time;
					ff->wr_date = f.wr_date;
					ff->size    = f.size;
					strcpyn(ff->name,szPathName,_MAX_PATH+_MAX_FNAME+1);
					m_FileList.Add(ff);
				}
			}
		}
		while(CFindFile::Next(&f));
	}

	if(bRecursive)
	{
		// pathname completo
		if((strlen(lpcszStartDir) + 3) < sizeof(szPathName))
			_snprintf(szPathName,sizeof(szPathName)-1,"%s*.*",lpcszStartDir);
		else
			return(INVALID_HANDLE_VALUE);
		
		// cerca la prima istanza dello skeleton
		if(CFindFile::First(szPathName,_A_SUBDIR,&f))
		{
			do
			{
				// cerca nelle subdir
				if((f.attrib & _A_SUBDIR) && f.name[0]!='.')
				{
					// pathname completo
					if((strlen(lpcszStartDir) + strlen(f.name) + 1) < sizeof(szPathName))
						_snprintf(szPathName,sizeof(szPathName)-1,"%s%s\\",lpcszStartDir,f.name);
					else
						return(INVALID_HANDLE_VALUE);
					
					// ricerca ricorsivamente
					CFindFile::Search(szPathName,lpcszSkel,bRecursive,uAttribute);
				}
			}
			while(CFindFile::Next(&f));
		}
	}

	return(f.handle);
}

/*
	First()

	Cerca la prima istanza del nome file.
	Il terzo parametro e' opzionale perche per la ricerca ricorsiva deve essere usata la struttura del chiamante, non quella della classe.
*/
BOOL CFindFile::First(LPCSTR lpcszFileName,UINT uAttribute,FINDFILE* f/* = NULL */)
{
	HANDLE hHandle = INVALID_HANDLE_VALUE;

	if(!f)
	{
		memset(&m_FindFile,'\0',sizeof(FINDFILE));
		m_FindFile.handle = INVALID_HANDLE_VALUE;
	}
	else
	{
		memset(f,'\0',sizeof(FINDFILE));
		f->handle = INVALID_HANDLE_VALUE;
	}

	if((hHandle = ::FindFirstFile(lpcszFileName,&m_FindData))!=INVALID_HANDLE_VALUE)
	{
		if(!f)
		{
			// struttura della classe
			strcpyn(m_FindFile.name,m_FindData.cFileName,sizeof(m_FindFile.name));
			m_FindFile.handle  = hHandle;
			m_FindFile.size    = m_FindData.nFileSizeLow;
			m_FindFile.wr_date = (UINT)(m_FindData.ftLastWriteTime.dwLowDateTime >> 16);
			m_FindFile.wr_time = (UINT)(m_FindData.ftLastWriteTime.dwLowDateTime & 0xffff);
			m_FindFile.attrib  = (UINT)m_FindData.dwFileAttributes;
		}
		else
		{
			// struttura del chiamante
			strcpyn(f->name,m_FindData.cFileName,_MAX_PATH+_MAX_FNAME+1);
			f->handle  = hHandle;
			f->size    = m_FindData.nFileSizeLow;
			f->wr_date = (UINT)(m_FindData.ftLastWriteTime.dwLowDateTime >> 16);
			f->wr_time = (UINT)(m_FindData.ftLastWriteTime.dwLowDateTime & 0xffff);
			f->attrib  = (UINT)m_FindData.dwFileAttributes;
		}
	}

	return(hHandle!=INVALID_HANDLE_VALUE);
}

/*
	Next()
	
	Cerca l'istanza successiva del nome file.
	Il terzo parametro e' opzionale perche per la ricerca ricorsiva deve essere usata la struttura del chiamante, non quella della classe.
*/
BOOL CFindFile::Next(FINDFILE* f/* = NULL */)
{
	HANDLE hHandle = (f!=(FINDFILE*)NULL ? f->handle : m_FindFile.handle);

	if(hHandle==INVALID_HANDLE_VALUE)
		return(FALSE);

	if(!::FindNextFile(hHandle,&m_FindData))
		return(FALSE);

	if(!f)
	{
		// struttura della classe
		strcpyn(m_FindFile.name,m_FindData.cFileName,sizeof(m_FindFile.name));
		m_FindFile.size    = m_FindData.nFileSizeLow;
		m_FindFile.wr_date = (UINT)(m_FindData.ftLastWriteTime.dwLowDateTime >> 16);
		m_FindFile.wr_time = (UINT)(m_FindData.ftLastWriteTime.dwLowDateTime & 0xffff);
		m_FindFile.attrib  = (UINT)m_FindData.dwFileAttributes;
	}
	else
	{
		// struttura del chiamante
		strcpyn(f->name,m_FindData.cFileName,_MAX_PATH+_MAX_FNAME+1);
		f->size    = m_FindData.nFileSizeLow;
		f->wr_date = (UINT)(m_FindData.ftLastWriteTime.dwLowDateTime >> 16);
		f->wr_time = (UINT)(m_FindData.ftLastWriteTime.dwLowDateTime & 0xffff);
		f->attrib  = (UINT)m_FindData.dwFileAttributes;
	}

	return(TRUE);
}

/*
	Close()
	
	Chiude la ricerca.
	Il terzo parametro e' opzionale perche per la ricerca ricorsiva deve essere usata la struttura del chiamante, non quella della classe.
*/
BOOL CFindFile::Close(HANDLE h)
{
	BOOL bClosed = FALSE;

	HANDLE hHandle = (h!=INVALID_HANDLE_VALUE ? h : m_FindFile.handle);

	if(hHandle!=INVALID_HANDLE_VALUE)
	{
		bClosed = ::FindClose(hHandle);
		m_FindFile.handle = INVALID_HANDLE_VALUE;
	}
	
	return(bClosed);
}

/*
	GetFileSize()

	Restituisce la dimensione del file.
*/
DWORD CFindFile::GetFileSize(LPCSTR lpcszFileName)
{
	DWORD dwSize = (DWORD)-1;
	HANDLE hHandle;

	if((hHandle = ::CreateFile(lpcszFileName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL))!=INVALID_HANDLE_VALUE)
	{
		dwSize = ::GetFileSize(hHandle,NULL);
		::CloseHandle(hHandle);
	}

	return(dwSize);
}

/*
	SetFileTime()
*/
BOOL CFindFile::SetFileTime(LPCSTR lpcszFileName,UINT uDate,UINT uTime)
{
	BOOL bSet = FALSE;
	HANDLE hHandle;
	FILETIME filetime;

	if((hHandle = ::CreateFile(lpcszFileName,GENERIC_READ|GENERIC_WRITE,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL))!=INVALID_HANDLE_VALUE)
	{
		if(::DosDateTimeToFileTime((WORD)uDate,(WORD)uTime,&filetime))
			bSet = ::SetFileTime(hHandle,NULL,NULL,&filetime);

		::CloseHandle(hHandle);
	}

	return(bSet);
}

/*
	GetFileTime()
*/
BOOL CFindFile::GetFileTime(LPCSTR lpcszFileName,UINT* lpuDate,UINT* lpuTime)
{
	BOOL bGet = FALSE;
	HANDLE hHandle;
	FILETIME filetime;

	if((hHandle = ::CreateFile(lpcszFileName,GENERIC_READ|GENERIC_WRITE,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL))!=INVALID_HANDLE_VALUE)
	{
		if(::GetFileTime(hHandle,NULL,NULL,&filetime))
			bGet = (BOOL)::FileTimeToDosDateTime(&filetime,(WORD*)lpuDate,(WORD*)lpuTime);

		::CloseHandle(hHandle);
	}

	return(bGet);
}

/*
	SplitPathname()
*/
void CFindFile::SplitPathname(LPCSTR lpcszPathName,LPSTR lpszDirectory,UINT uDirSize,LPSTR lpszFileName,UINT uFileSize)
{
	char* p;
	
	// controlla se il pathname contiene una directory
	if((p = strchr(lpcszPathName,'\\'))==(char*)NULL)
		p = strchr(lpcszPathName,':');

	if(p)
	{
		char szBuffer[_MAX_PATH+1];
		
		strcpyn(szBuffer,lpcszPathName,sizeof(szBuffer));
		strrev(szBuffer);
		
		if((p = strchr(szBuffer,'\\'))==(char*)NULL)
			p = strchr(szBuffer,':');

		strcpyn(lpszDirectory,p,uDirSize);
		strrev(lpszDirectory);
		*p = '\0';
		
		strcpyn(lpszFileName,szBuffer,uFileSize);
		strrev(lpszFileName);
	}
	else // nessuna directory, ricava la corrente
	{
		::GetCurrentDirectory(uDirSize,lpszDirectory);
		strcpyn(lpszFileName,lpcszPathName,uFileSize);
	}

	if((lpszDirectory[strlen(lpszDirectory)-1]!='\\') && (strlen(lpszDirectory)+1 < uDirSize))
		strcat(lpszDirectory,"\\");
}

/*
	CreatePathname()
*/
BOOL CFindFile::CreatePathname(LPCSTR lpcszPathName)
{    
	char* p;
	FINDFILE* f;
	CFindFileList* filelist;
	char szDrive[_MAX_DRIVE+1+1];
	char szDirectory[_MAX_PATH+1];
	DWORD dwAttribute = 0L;

	// non crea il pathname se gia' esiste, se si tratta di un file restituisce errore
	if((dwAttribute = ::GetFileAttributes(lpcszPathName))!=0xFFFFFFFF)
		return((dwAttribute & FILE_ATTRIBUTE_DIRECTORY) ? TRUE : FALSE);

	strcpyn(szDirectory,lpcszPathName,sizeof(szDirectory));
	::EnsureBackslash(szDirectory,sizeof(szDirectory));
		
	memset(szDrive,'\0',sizeof(szDrive));
	if((p = strchr(szDirectory,':'))!=(char*)NULL)
	{
		szDrive[0] = *(p-1);
		szDrive[1] = *(p);
		
		if(*(p+1)=='\\')
		{
			szDrive[2] = '\\';
			szDrive[3] = '\0';
		}
		else
			szDrive[2] = '\0';
	}
	else
	{
		// imposta il drive solo per i pathname assoluti
		if(szDirectory[0]=='\\')
		{
			char szBuffer[_MAX_PATH+1];
			if(::GetCurrentDirectory(sizeof(szBuffer)-1,szBuffer)!=0)
			{
				szDrive[0] = szBuffer[0];
				szDrive[1] = ':';
				szDrive[2] = '\\';
				szDrive[3] = '\0';
			}
			else
				return(FALSE);
		}
	}
	
	// rimuove l'identificatore del drive
	strrev(szDirectory);
	if((p = strchr(szDirectory,':'))!=(char*)NULL)
		*p = '\0';
	strrev(szDirectory);

	// crea la lista con le directory contenute nel pathname
	if((filelist = new CFindFileList())==(CFindFileList*)NULL)
		return(FALSE);

	// il pathname deve contenere almeno una directory
	if(strchr(szDirectory,'\\')!=(char*)NULL)
	{
		char* token = strtok(szDirectory,"\\");
		
		for(int i = 0; token!=(char*)NULL; i++)
		{
			if((f = new FINDFILE)!=(FINDFILE*)NULL)
			{
				strcpyn(f->name,token,_MAX_PATH+_MAX_FNAME+1);
				filelist->Add(f);
				token = strtok((char*)NULL,"\\");
			}
		}
	}

	// identificativo del drive
	if(*szDrive)
		strcpyn(szDirectory,szDrive,sizeof(szDirectory));
	
	// crea il pathname
	for(int i = 0; i < filelist->Count(); i++)
	{
		if((f = (FINDFILE*)filelist->GetAt(i))!=(FINDFILE*)NULL)
		{
			strcat(szDirectory,f->name);
			if(::GetFileAttributes(szDirectory)==0xFFFFFFFF)
				::CreateDirectory(szDirectory,NULL);
			strcat(szDirectory,"\\");
		}
	}

	if(filelist)
		delete filelist;

	return(::GetFileAttributes(szDirectory)!=0xFFFFFFFF);
}
