/*
	CHttp.cpp
	Classe base per la struttura per la connessione HTTP.
	Luca Piergentili, 05/10/00
	lpiergentili@yahoo.com
	http://www.geocities.com/lpiergentili/
*/
#include "env.h"
#include "pragma.h"
#include "window.h"
#include <stdio.h>
#include <stdlib.h>
#include "stristr.h"
#include <ctype.h>
#include "strcpyn.h"
#include "CSock.h"
#include "CHttp.h"
#include "CTextFile.h"

/*
	Reset()

	Resetta i dati relativi alla connessione.
*/
void CHttpConnection::Reset(void)
{
	// resetta il socket
	CSock::Reset();

	// resetta i valori interni
	m_nSocketCode = 0;
	memset(m_szSocketCode,'\0',sizeof(m_szSocketCode));
	m_nHttpCode = (int)HTTP_STATUS_VOID;
	memset(m_szHttpCode,'\0',sizeof(m_szHttpCode));
	memset(&m_HttpObject,'\0',sizeof(HTTPOBJECT));
	m_HttpHeaderList.DeleteAll();
	memset(m_szServerName,'\0',sizeof(m_szServerName));
	memset(m_szHostName,'\0',sizeof(m_szHostName));
	memset(m_szHostAddr,'\0',sizeof(m_szHostAddr));
	memset(&m_Proxy,'\0',sizeof(PROXY));
	m_bSkipExisting = FALSE;
	m_nConnectionRetry = 1;
	m_bAreCookiesAllowed = FALSE;
	memset(m_szCookieFileName,'\0',sizeof(m_szCookieFileName));
	m_nMinSize = m_nMaxSize = 0;
}

/*
	Close()

	Chiude la connessione.
*/
BOOL CHttpConnection::Close(void)
{
	// chiude il socket
	BOOL bClosed = CSock::Close();

	// aggiorna i cookies
	if(CookiesAllowed())
		UpdateCookies();

	return(bClosed);
}

/*
	ParseCookie()

	Estrae il cookie dalla stringa (ricevuta dall'host) per formattarlo nei campi della struttura.
*/
void CHttpConnection::ParseCookie(LPCSTR lpcszCookie,COOKIE* cookie)
{
	int i;
	char* p = (char*)lpcszCookie;

	memset(cookie,'\0',sizeof(COOKIE));

	// name
	while(*p && *p==' ')
		p++;
	for(i = 0; *p && *p!='=' && i < COOKIE_NAME_LEN;)
		cookie->name[i++] = *p++;

	// value
	while(*p && (*p==' ' || *p=='='))
		p++;
	for(i = 0; *p && *p!=';' && i < COOKIE_VALUE_LEN;)
		cookie->value[i++] = *p++;

	// expires
	p = stristr(lpcszCookie,"expires");
	if(p)
	{
		p += 7;
		while(*p && (*p==' ' || *p=='='))
			p++;
		
		for(i = 0; *p && *p!=';' && i < COOKIE_EXPIRES_LEN;)
			cookie->expires[i++] = *p++;
	}
	else
		p = (char*)lpcszCookie;

	// path
	p = stristr(lpcszCookie,"path");
	if(p)
	{
		p += 4;
		while(*p && (*p==' ' || *p=='='))
			p++;
		
		for(i = 0; *p && *p!=';' && i < COOKIE_PATH_LEN;)
			cookie->path[i++] = *p++;
	}
	else
		p = (char*)lpcszCookie;

	// domain
	p = stristr(lpcszCookie,"domain");
	if(p)
	{
		p += 6;
		while(*p && (*p==' ' || *p=='='))
			p++;
		
		for(i = 0; *p && *p!=';' && i < COOKIE_DOMAIN_LEN;)
			cookie->domain[i++] = *p++;

		char* pHost = strstr(cookie->domain,"://");
		if(pHost)
		{
			pHost += 3;
			int i = strlen(pHost);
			memmove(cookie->domain,pHost,i);
			cookie->domain[i] = '\0';
		}
	}
	else
		p = (char*)lpcszCookie;

	// secure
	while(*p && (*p==' ' || *p==';'))
		p++;
	if(p)
	{
		if(isdigit(*p))
			cookie->secure = atoi(p);
	}
}

/*
	ParseInternalCookie()

	Estrae il cookie dalla stringa (letta dal file) per formattarlo nei campi della struttura.
	Utilizzata internamente dato che la stringa non contiene il cookie nel formato standard (ricevuto
	dall'host), ma in quello interno (letto dal file).
*/
BOOL CHttpConnection::ParseInternalCookie(LPCSTR lpcszCookie,COOKIE* cookie)
{
	int i,n;

	if(lpcszCookie[0]=='?')
		return(FALSE);

	memset(cookie,'\0',sizeof(COOKIE));

	// domain
	for(i = 0,n = 0; lpcszCookie[i] && lpcszCookie[i]!=' ' && lpcszCookie[i]!=';' && n < COOKIE_DOMAIN_LEN; i++,n++)
		cookie->domain[n] = lpcszCookie[i];

	// path
	for(; lpcszCookie[i] && (lpcszCookie[i]==' ' || lpcszCookie[i]==';');)
		i++;
	for(n = 0; lpcszCookie[i] && lpcszCookie[i]!=' ' && lpcszCookie[i]!=';' && n < COOKIE_PATH_LEN; i++,n++)
		cookie->path[n] = lpcszCookie[i];
	
	// name
	for(; lpcszCookie[i] && (lpcszCookie[i]==' ' || lpcszCookie[i]==';');)
		i++;
	for(n = 0; lpcszCookie[i] && lpcszCookie[i]!=' ' && lpcszCookie[i]!='=' && n < COOKIE_NAME_LEN; i++,n++)
		cookie->name[n] = lpcszCookie[i];
	
	// value
	for(; lpcszCookie[i] && (lpcszCookie[i]==' ' || lpcszCookie[i]=='=');)
		i++;
	for(n = 0; lpcszCookie[i] && lpcszCookie[i]!=' ' && lpcszCookie[i]!=';' && n < COOKIE_VALUE_LEN; i++,n++)
		cookie->value[n] = lpcszCookie[i];

	// expires
	for(; lpcszCookie[i] && (lpcszCookie[i]==' ' || lpcszCookie[i]==';');)
		i++;
	for(n = 0; lpcszCookie[i] && lpcszCookie[i]!=';' && n < COOKIE_EXPIRES_LEN; i++,n++)
		cookie->expires[n] = lpcszCookie[i];

	// secure
	for(; lpcszCookie[i] && (lpcszCookie[i]==' ' || lpcszCookie[i]==';');)
		i++;
	cookie->secure = atoi(lpcszCookie+i);

	return(TRUE);
}

/*
	SetCookie()

	Registra il cookie nel file.
	Effettua la risoluzione del nome dell'host per ricavare il nome reale del dominio.
*/
BOOL CHttpConnection::SetCookie(COOKIE* cookie)
{
	CTextFile file;
	DWORD dwWritten = FILE_EOF;
	
	// apre il file per i cookies
	if(file.Open(m_szCookieFileName))
	{
		file.GoBottom();

		// domain - ricava il nome reale dell'host
		if(cookie->domain[0]=='\0')
			strcpyn(cookie->domain,GetHostName(),COOKIE_DOMAIN_LEN+1);

		char szHost[HOSTNAME_SIZE+1];
		strcpyn(szHost,cookie->domain,sizeof(szHost));
		char* pHost = strstr(szHost,"://");
		if(pHost)
		{
			pHost += 3;
			int n = strlen(pHost);
			memmove(szHost,pHost,n);
			szHost[n] = '\0';
		}
		if(isdigit(szHost[0]))
		{
			strcpyn(szHost,GetHostByAddr(szHost),sizeof(szHost));
		}
		else
		{
			strcpyn(szHost,GetHostByName(szHost),sizeof(szHost));
			strcpyn(szHost,GetHostByAddr(szHost),sizeof(szHost));
		}
		strcpyn(cookie->domain,szHost,COOKIE_DOMAIN_LEN+1);

		// registra il cookie nel file
		dwWritten = file.WriteFormattedLine("%s; %s; %s=%s; %s; %d",
									cookie->domain,
									cookie->path[0]    != '\0' ? cookie->path    : "?",
									cookie->name[0]    != '\0' ? cookie->name    : "?",
									cookie->value[0]   != '\0' ? cookie->value   : "?",
									cookie->expires[0] != '\0' ? cookie->expires : "?",
									cookie->secure
									);
		file.Close();
	}

	return(dwWritten!=FILE_EOF);
}

/*
	GetCookies()

	Recupera i cookies dal file.
	Effettua la risoluzione del nome dell'host per ricavare il nome reale del dominio.
*/
int CHttpConnection::GetCookies(LPCSTR lpcszHost,LPCSTR lpcszPath)
{
	CTextFile file;
	int nCookies = 0;
	COOKIE cookie;
	char szBuffer[COOKIE_MAX_LEN+32];

	// elimina tutti i cookies dalla lista per i campi dell'header HTTP
	DeleteHeader("Cookie");

	// apre il file per i cookies
	if(file.Open(m_szCookieFileName))
	{
		file.GoTop();
		
		while(file.ReadLine(szBuffer,sizeof(szBuffer)-1)!=FILE_EOF)
		{
			// carica il cookie nella struttura interna
			if(!ParseInternalCookie(szBuffer,&cookie))
				continue;

			// domain - ricava il nome reale dell'host
			char szHost[HOSTNAME_SIZE+1];
			strcpyn(szHost,lpcszHost,sizeof(szHost));
			char* pHost = strstr(szHost,"://");
			if(pHost)
			{
				pHost += 3;
				int n = strlen(pHost);
				memmove(szHost,pHost,n);
				szHost[n] = '\0';
			}
			if(isdigit(szHost[0]))
			{
				strcpyn(szHost,GetHostByAddr(szHost),sizeof(szHost));
			}
			else
			{
				strcpyn(szHost,GetHostByName(szHost),sizeof(szHost));
				strcpyn(szHost,GetHostByAddr(szHost),sizeof(szHost));
			}

			// confronta dominio/path
			if(strstr(szHost,cookie.domain))
			{
				if(strstr(lpcszPath,cookie.path))
				{
					// aggiunge il cookie alla lista per i campi dell'header HTTP
					_snprintf(szBuffer,sizeof(szBuffer)-1,"%s=%s",cookie.name,cookie.value);
					AddHeader("Cookie",szBuffer);
					nCookies++;
				}
			}
		}
		
		file.Close();
	}
	
	return(nCookies);
}

/*
	UpdateCookies()

	Aggiorna il contenuto del file per i cookies.
*/
BOOL CHttpConnection::UpdateCookies(void)
{
	CTextFile file;
	COOKIE cookie;
	char szBuffer[COOKIE_MAX_LEN+32];

	// elimina tutti i cookies dalla lista per i campi dell'header HTTP
	DeleteHeader("Cookie");

	// apre il file per i cookies
	if(file.Open(m_szCookieFileName,/*bCreateIfNotExist*/FALSE,/*dwAccessMode*/GENERIC_READ|GENERIC_WRITE,/*dwShareMode*/FILE_EXCLUSIVE))
	{
		file.GoTop();
		
		while(file.ReadLine(szBuffer,sizeof(szBuffer)-1)!=FILE_EOF)
		{
			// carica il cookie nella struttura interna
			if(!ParseInternalCookie(szBuffer,&cookie))
				continue;

			// controlla se deve eliminare il cookie
			if(IsCookieExpired(cookie.expires))
			{
				szBuffer[0] = '?';
				strcat(szBuffer,"\r\n");
				file.Seek(strlen(szBuffer)*-1,FILE_CURRENT);
				file.Write(szBuffer,strlen(szBuffer));
			}
		}
		file.Close();
	}
	
	return(TRUE);
}

/*
	IsCookieExpired()

	Verifica se il cookie e' scaduto o meno.
*/
BOOL CHttpConnection::IsCookieExpired(LPCSTR lpcszDate)
{
	BOOL bExpired = TRUE;

	if(lpcszDate[0]!='?')
	{
		CDateTime currentDate(ANSI_SHORT);
		CDateTime cookieDate(GMT_SHORT);
		cookieDate.ConvertDate(GMT_SHORT,ANSI_SHORT,lpcszDate,NULL);

		if(cookieDate.GetYear() > currentDate.GetYear())
		{
			bExpired = FALSE;
		}
		else if(cookieDate.GetYear() < currentDate.GetYear())
		{
			bExpired = TRUE;
		}
		else if(cookieDate.GetYear()==currentDate.GetYear())
		{
			if(cookieDate.GetMonth() > currentDate.GetMonth())
			{
				bExpired = FALSE;
			}
			else if(cookieDate.GetMonth() < currentDate.GetMonth())
			{
				bExpired = TRUE;
			}
			else if(cookieDate.GetMonth()==currentDate.GetMonth())
			{
				if(cookieDate.GetDay() > currentDate.GetDay())
				{
					bExpired = FALSE;
				}
				else if(cookieDate.GetDay() < currentDate.GetDay())
				{
					bExpired = TRUE;
				}
				else if(cookieDate.GetDay()==currentDate.GetDay())
				{
					//$ controllare l'ora
					bExpired = FALSE;
				}
			}
		}
	}

	return(bExpired);
}

/*
	EnumCookies()
*/
HTTPHEADER* CHttpConnection::EnumCookies(void)
{
	static int i = -1;
	HTTPHEADER* httpheader = NULL;
	ITERATOR iter;

next:
	if(++i >= m_HttpHeaderList.Count())
	{
		i = -1;
		httpheader = NULL;
	}
	else
	{
		if((iter = m_HttpHeaderList.FindAt(i))!=(ITERATOR)NULL)
		{
			httpheader = (HTTPHEADER*)iter->data;
			if(strcmp(httpheader->name,"Cookie")!=0)
			{
				httpheader = NULL;
				goto next;
			}
		}
	}

	return(httpheader);
}

/*
	EnumHeaders()
*/
HTTPHEADER* CHttpConnection::EnumHeaders(void)
{
	static int i = -1;
	HTTPHEADER* httpheader = NULL;
	ITERATOR iter;

next:
	if(++i >= m_HttpHeaderList.Count())
	{
		i = -1;
		httpheader = NULL;
	}
	else
	{
		if((iter = m_HttpHeaderList.FindAt(i))!=(ITERATOR)NULL)
		{
			httpheader = (HTTPHEADER*)iter->data;
			if(strcmp(httpheader->name,"Cookie")==0)
			{
				httpheader = NULL;
				goto next;
			}
		}
	}

	return(httpheader);
}

/*
	AddHeader()
*/
BOOL CHttpConnection::AddHeader(LPCSTR lpcszName,LPCSTR lpcszValue,BOOL bAllowDuplicates/*=TRUE*/)
{
	HTTPHEADER* httpheader;
	ITERATOR iter;
	BOOL bFound = FALSE;

	if(!bAllowDuplicates)
		if((iter = m_HttpHeaderList.First())!=(ITERATOR)NULL)
			while(iter!=(ITERATOR)NULL)
			{
				httpheader = (HTTPHEADER*)iter->data;

				if(strcmp(lpcszName,httpheader->name)==0)
					if(strcmp(lpcszValue,httpheader->value)==0)
					{
						bFound = TRUE;
						break;
					}

				iter = m_HttpHeaderList.Next(iter);
			}

	if(!bFound)
	{
		httpheader = new HTTPHEADER;
		strcpyn(httpheader->name,lpcszName,HTTPHEADER_NAME_LEN+1);
		strcpyn(httpheader->value,lpcszValue,HTTPHEADER_VALUE_LEN+1);
		m_HttpHeaderList.Add(httpheader);
	}

	return(!bFound);
}

/*
	DeleteHeader()
*/
void CHttpConnection::DeleteHeader(LPCSTR lpcszName)
{
	HTTPHEADER* httpheader;
	ITERATOR iter;

	if((iter = m_HttpHeaderList.First())!=(ITERATOR)NULL)
		while(iter!=(ITERATOR)NULL)
		{
			httpheader = (HTTPHEADER*)iter->data;

			if(strcmp(lpcszName,httpheader->name)==0)
				m_HttpHeaderList.Delete(iter);

			iter = m_HttpHeaderList.Next(iter);
		}
}

/*
	GetErrorString()

	Restituisce la stringa relativa al codice d'errore (HTTO o socket).
*/
LPCSTR CHttpConnection::GetErrorString(void)
{
	char* p;
	if(IsSocketError())
		p = m_szSocketCode;
	else if(IsHttpError())
		p = m_szHttpCode;
	else
		p = "no error";
	return(p);
}

/*
	ResetSocketError()
*/
void CHttpConnection::ResetSocketError(void)
{
	m_nSocketCode = 0;
	memset(m_szSocketCode,'\0',sizeof(m_szSocketCode));
}

/*
	ResetHttpError()
*/
void CHttpConnection::ResetHttpError(void)
{
	m_nHttpCode = (int)HTTP_STATUS_VOID;
	memset(m_szHttpCode,'\0',sizeof(m_szHttpCode));
}

/*
	SetSocketError()

	Imposta il codice d'errore per il socket (numerico/descrizione).
*/
void CHttpConnection::SetSocketError(void)
{
	if(m_nSocketCode==0 && m_szSocketCode[0]=='\0')
	{
		m_nSocketCode = CSock::GetWSAErrorNumber();
		strcpyn(m_szSocketCode,CSock::GetWSAErrorString(),sizeof(m_szSocketCode));
	}
}
