/*
	CLogDatabase.cpp
	Implementazione della classe relativa alle tabella.
	Luca Piergentili, 26/06/00
	lpiergentili@yahoo.com
	http://www.geocities.com/lpiergentili/
*/
#include "env.h"
#include "pragma.h"
#include "typedef.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "strings.h"
#include "strcpyn.h"
#include "strsetn.h"
#include "window.h"
#include "CMutexEx.h"
#include "CBase.h"
#include "CTable.h"
#include "CLogDatabase.h"

/*
	CLogTable()
*/
CLogTable::CLogTable(LPCSTR lpcszTableName/*=NULL*/,LPCSTR lpcszDataPath/*=NULL*/,BOOL bOpenTable/*=FALSE*/)
{
	// nome tabella
	strcpyn(m_szTableName,lpcszTableName ? lpcszTableName : LOG_TABLE,sizeof(m_szTableName));

	// pathname per tabella/indici
	if(lpcszDataPath)
	{
		_snprintf(m_szTablePath,sizeof(m_szTablePath)-1,			"%s\\%s.db",			lpcszDataPath,m_szTableName);
		_snprintf(m_szIndexIp,sizeof(m_szIndexIp)-1,				"%s\\%s.ip.idx",		lpcszDataPath,m_szTableName);
		_snprintf(m_szIndexGet,sizeof(m_szIndexGet)-1,			"%s\\%s.get.idx",		lpcszDataPath,m_szTableName);
		_snprintf(m_szIndexReferer,sizeof(m_szIndexReferer)-1,		"%s\\%s.referer.idx",	lpcszDataPath,m_szTableName);
		_snprintf(m_szIndexUserAgent,sizeof(m_szIndexUserAgent)-1,	"%s\\%s.useragent.idx",  lpcszDataPath,m_szTableName);
		_snprintf(m_szIndexDate,sizeof(m_szIndexDate)-1,			"%s\\%s.date.idx",		lpcszDataPath,m_szTableName);
	}
	else
	{
		_snprintf(m_szTablePath,sizeof(m_szTablePath)-1,			".\\%s.db",			m_szTableName);
		_snprintf(m_szIndexIp,sizeof(m_szIndexIp)-1,				".\\%s.ip.idx",		m_szTableName);
		_snprintf(m_szIndexGet,sizeof(m_szIndexGet)-1,			".\\%s.get.idx",		m_szTableName);
		_snprintf(m_szIndexReferer,sizeof(m_szIndexReferer)-1,		".\\%s.referer.idx",	m_szTableName);
		_snprintf(m_szIndexUserAgent,sizeof(m_szIndexUserAgent)-1,	".\\%s.useragent.idx",	m_szTableName);
		_snprintf(m_szIndexDate,sizeof(m_szIndexDate)-1,			".\\%s.date.idx",		m_szTableName);
	}

	// definisce la tabella
	static CBASE_TABLE table[] = {
		{"IP",		'C',	LOG_IP_LEN,		0},
		{"DATE",		'C',	LOG_DATE_LEN,		0},
		{"GET",		'C',	LOG_GET_LEN,		0},
		{"CODE",		'N',	LOG_CODE_LEN,		0},
		{"SIZE",		'N',	LOG_SIZE_LEN,		0},
		{"REFERER",	'C',	LOG_REFERER_LEN,	0},
		{"USERAGENT",	'C',	LOG_USERAGENT_LEN,	0},
		{NULL,		 0,	0,				0}
	};

	// definisce l'indice
	static CBASE_INDEX idx[] = {
		{NULL,	"IDX_IP",			"IP"},
		{NULL,	"IDX_GET",		"GET"},
		{NULL,	"IDX_REFERER",		"REFERER"},
		{NULL,	"IDX_USERAGENT",	"USERAGENT"},
		{NULL,	"IDX_DATE",		"DATE"},
		{NULL,	 NULL,			 NULL}
	};
	
	idx[0].file = m_szIndexIp;
	idx[1].file = m_szIndexGet;
	idx[2].file = m_szIndexReferer;
	idx[3].file = m_szIndexUserAgent;
	idx[4].file = m_szIndexDate;

	// crea l'oggetto per la tabella (derivato da CTable)
	m_pCBase = new CBase(m_szTablePath,&table[0],&idx[0]);

	// crea l'oggetto per il lock della tabella (derivato da CTable)
	char szMutexName[_MAX_PATH+1];
	_snprintf(szMutexName,sizeof(szMutexName)-1,"%s.%s",AfxGetAppName(),m_szTableName);
	m_TableMutex = new CMutexEx(szMutexName);

	// puntatore alla struttura della tabella
	table_struct = &table[0];

	// puntatore alla struttura dell'indice
	idx_struct = &idx[0];

	// azzera il record
	ResetMemvars();
	
	// apre la tabella
	if(bOpenTable)
	{
		if(m_pCBase->Open())
			m_pCBase->SetIndex(0);
	}
}

/*
	~CLogTable()
*/
CLogTable::~CLogTable()
{
	if(m_pCBase)
		delete m_pCBase,m_pCBase = NULL;
	if(m_TableMutex)
		delete m_TableMutex,m_TableMutex = NULL;
}

/*
	GetRecordAsString()
*/
const char* CLogTable::GetRecordAsString(void)
{
	if(strcmp(record.ip,"")==0)
		strsetn(record.ip,' ',sizeof(record.ip));
	if(strcmp(record.date,"")==0)
		strsetn(record.date,' ',sizeof(record.date));
	if(strcmp(record.get,"")==0)
		strsetn(record.get,' ',sizeof(record.get));
	if(strcmp(record.referer,"")==0)
		strsetn(record.referer,' ',sizeof(record.referer));
	if(strcmp(record.useragent,"")==0)
		strsetn(record.useragent,' ',sizeof(record.useragent));

	_snprintf(m_szRecord,
			sizeof(m_szRecord)-1,
			"%s;%s;%s;%d;%ld;%s;%s",
			record.ip,
			record.date,
			record.get,
			record.code,
			record.size,
			record.referer,
			record.useragent
			);

	return(m_szRecord);
}

/*
	GatherMemvars()
*/
void CLogTable::GatherMemvars(void)
{
	// passa le variabili di memoria al record
	m_pCBase->PutField(0,record.ip);
	m_pCBase->PutField(1,record.date);
	m_pCBase->PutField(2,record.get);
	m_pCBase->PutField(3,record.code);
	m_pCBase->PutField(4,record.size);
	m_pCBase->PutField(5,record.referer);
	m_pCBase->PutField(6,record.useragent);
}

/*
	ScatterMemvars()
*/
void CLogTable::ScatterMemvars(BOOL bTrim)
{
	// passa il record alle variabili di memoria
	              m_pCBase->GetField(0,record.ip,sizeof(record.ip)-1);
	              m_pCBase->GetField(1,record.date,sizeof(record.date)-1);
	              m_pCBase->GetField(2,record.get,sizeof(record.get)-1);
	record.code = m_pCBase->GetField(3,record.code);
	record.size = m_pCBase->GetField(4,record.size);
	              m_pCBase->GetField(5,record.referer,sizeof(record.referer)-1);
	              m_pCBase->GetField(6,record.useragent,sizeof(record.useragent)-1);

	if(bTrim)
	{
		strrtrim(record.ip,sizeof(record.ip)-1);
		strrtrim(record.date,sizeof(record.date)-1);
		strrtrim(record.get,sizeof(record.get)-1);
		strrtrim(record.referer,sizeof(record.referer)-1);
		strrtrim(record.useragent,sizeof(record.useragent)-1);
	}
}


/*
	CReportTable()
*/
CReportTable::CReportTable(LPCSTR lpcszTableName/*=NULL*/,LPCSTR lpcszDataPath/*=NULL*/,BOOL bOpenTable/*=FALSE*/)
{
	// nome tabella
	strcpyn(m_szTableName,lpcszTableName ? lpcszTableName : REPORT_TABLE,sizeof(m_szTableName));

	// pathname per tabella/indici
	if(lpcszDataPath)
	{
		_snprintf(m_szTablePath,sizeof(m_szTablePath)-1,			"%s\\%s.db",			lpcszDataPath,m_szTableName);
		_snprintf(m_szIndexTot,sizeof(m_szIndexTot)-1,			"%s\\%s.tot.idx",		lpcszDataPath,m_szTableName);
		_snprintf(m_szIndexRate,sizeof(m_szIndexRate)-1,			"%s\\%s.rate.idx",		lpcszDataPath,m_szTableName);
		_snprintf(m_szIndexData,sizeof(m_szIndexData)-1,			"%s\\%s.data.idx",		lpcszDataPath,m_szTableName);
		_snprintf(m_szIndexDate,sizeof(m_szIndexDate)-1,			"%s\\%s.date.idx",		lpcszDataPath,m_szTableName);
	}
	else
	{
		_snprintf(m_szTablePath,sizeof(m_szTablePath)-1,			".\\%s.db",			m_szTableName);
		_snprintf(m_szIndexTot,sizeof(m_szIndexTot)-1,			".\\%s.tot.idx",		m_szTableName);
		_snprintf(m_szIndexRate,sizeof(m_szIndexRate)-1,			".\\%s.rate.idx",		m_szTableName);
		_snprintf(m_szIndexData,sizeof(m_szIndexData)-1,			".\\%s.data.idx",		m_szTableName);
		_snprintf(m_szIndexDate,sizeof(m_szIndexDate)-1,			".\\%s.date.idx",		m_szTableName);
	}

	// definisce la tabella
	static CBASE_TABLE table[] = {
		{"TOT",		'N',	REPORT_TOT_LEN,	0},
		{"RATE",		'C',	REPORT_RATE_LEN,	0},
		{"DATA",		'C',	REPORT_DATA_LEN,	0},
		{"DATE",		'C',	REPORT_DATE_LEN,	0},
		{NULL,		 0,	0,				0}
	};

	// definisce l'indice
	static CBASE_INDEX idx[] = {
		{NULL,	"IDX_TOT",	"TOT"},
		{NULL,	"IDX_RATE",	"RATE"},
		{NULL,	"IDX_DATA",	"DATA"},
		{NULL,	"IDX_DATE",	"DATE"},
		{NULL,	 NULL,		 NULL}
	};
	
	idx[0].file = m_szIndexTot;
	idx[1].file = m_szIndexRate;
	idx[2].file = m_szIndexData;
	idx[3].file = m_szIndexDate;

	// crea l'oggetto per la tabella (derivato da CTable)
	m_pCBase = new CBase(m_szTablePath,&table[0],&idx[0]);

	// crea l'oggetto per il lock della tabella (derivato da CTable)
	char szMutexName[_MAX_PATH+1];
	_snprintf(szMutexName,sizeof(szMutexName)-1,"%s.%s",AfxGetAppName(),m_szTableName);
	m_TableMutex = new CMutexEx(szMutexName);

	// puntatore alla struttura della tabella
	table_struct = &table[0];

	// puntatore alla struttura dell'indice
	idx_struct = &idx[0];

	// azzera il record
	ResetMemvars();
	
	// apre la tabella
	if(bOpenTable)
	{
		if(m_pCBase->Open())
			m_pCBase->SetIndex(0);
	}
}

/*
	~CReportTable()
*/
CReportTable::~CReportTable()
{
	if(m_pCBase)
		delete m_pCBase,m_pCBase = NULL;
	if(m_TableMutex)
		delete m_TableMutex,m_TableMutex = NULL;
}

/*
	GetRecordAsString()
*/
const char* CReportTable::GetRecordAsString(void)
{
	if(strcmp(record.data,"")==0)
		strsetn(record.data,' ',sizeof(record.data));

	_snprintf(m_szRecord,
			sizeof(m_szRecord)-1,
			"%ld;%0.5f;%s;%s",
			record.tot,
			record.rate,
			record.data,
			record.date
			);

	return(m_szRecord);
}

/*
	GatherMemvars()
*/
void CReportTable::GatherMemvars(void)
{
	// passa le variabili di memoria al record
	m_pCBase->PutField(0,record.tot);
	m_pCBase->PutField(1,record.rate);
	m_pCBase->PutField(2,record.data);
	m_pCBase->PutField(3,record.date);
}

/*
	ScatterMemvars()
*/
void CReportTable::ScatterMemvars(BOOL bTrim)
{
	// passa il record alle variabili di memoria
	record.tot = m_pCBase->GetField(0,record.tot);
	             m_pCBase->GetField(1,record.rate,sizeof(record.rate)-1);
	             m_pCBase->GetField(2,record.data,sizeof(record.data)-1);
	             m_pCBase->GetField(3,record.date,sizeof(record.date)-1);

	if(bTrim)
	{
		strrtrim(record.data,sizeof(record.data)-1);
		strrtrim(record.date,sizeof(record.date)-1);
	}
}
