/*
	CLogDatabaseService.cpp
	Implementazione della classe relativa alle operazioni sulla tabella.
	Luca Piergentili, 02/06/00
	lpiergentili@yahoo.com
	http://www.geocities.com/lpiergentili/
*/
#include "env.h"
#include "pragma.h"
#include "typedef.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "strcpyn.h"
#include "strings.h"
#include "window.h"
#include "CDateTime.h"
#include "CLogDatabase.h"
#include "CLogDatabaseService.h"

/*
	CLogDatabaseService()
*/
CLogDatabaseService::CLogDatabaseService(LPCSTR lpcszTableName/*=NULL*/,LPCSTR lpcszDataPath/*=NULL*/,BOOL bLockTable/*=FALSE*/)
{
	m_bLockTable = bLockTable;
	m_LogTable   = new CLogTable(lpcszTableName,lpcszDataPath);
	m_bOpened    = m_LogTable ? m_LogTable->Open() : FALSE;
}

/*
	~CLogDatabaseService()
*/
CLogDatabaseService::~CLogDatabaseService()
{
	if(m_LogTable)
	{
		if(m_bOpened)
			m_LogTable->Close();
		delete m_LogTable,m_LogTable = (CLogTable*)NULL;
	}
}

/*
	Insert()
*/
BOOL CLogDatabaseService::Insert(LPCSTR lpcszIp,LPCSTR lpcszDate,LPCSTR lpcszGet,UINT nCode,UINT nSize,LPCSTR lpcszReferer,LPCSTR lpcszUserAgent)
{
	BOOL bInserted = FALSE;

	if(LockTable())
	{
		m_LogTable->ResetMemvars();
		m_LogTable->PutField_Ip(lpcszIp);
		m_LogTable->PutField_Date(lpcszDate);
		m_LogTable->PutField_Get(lpcszGet);
		m_LogTable->PutField_Code(nCode);
		m_LogTable->PutField_Size(nSize);
		m_LogTable->PutField_Referer(lpcszReferer);
		m_LogTable->PutField_UserAgent(lpcszUserAgent);
		m_LogTable->GatherMemvars();
		bInserted = m_LogTable->Insert();

		UnlockTable();
	}

	return(bInserted);
}

/*
	Count()
*/
UINT CLogDatabaseService::Count(void)
{
	UINT nTot = 0;

	if(LockTable())
	{
		if(m_LogTable->GoTop())
			while(!m_LogTable->Eof())
			{
				m_LogTable->ScatterMemvars();
				nTot++;
				m_LogTable->GetNext();
			}

		UnlockTable();
	}

	return(nTot);
}

/*
	LockTable()
*/
BOOL CLogDatabaseService::LockTable(void)
{
	return(m_bOpened ? (m_bLockTable ? m_LogTable->LockTable() : TRUE) : FALSE);
}

/*
	UnlockTable()
*/
void CLogDatabaseService::UnlockTable(void)
{
	if(m_bOpened && m_bLockTable)
		m_LogTable->UnlockTable();
}


/*
	CReportDatabaseService()
*/
CReportDatabaseService::CReportDatabaseService(LPCSTR lpcszTableName/*=NULL*/,LPCSTR lpcszDataPath/*=NULL*/,BOOL bLockTable/*=FALSE*/)
{
	m_bLockTable  = bLockTable;
	m_ReportTable = new CReportTable(lpcszTableName,lpcszDataPath);
	m_bOpened     = m_ReportTable ? m_ReportTable->Open() : FALSE;
}

/*
	~CReportDatabaseService()
*/
CReportDatabaseService::~CReportDatabaseService()
{
	if(m_ReportTable)
	{
		if(m_bOpened)
			m_ReportTable->Close();
		delete m_ReportTable,m_ReportTable = (CReportTable*)NULL;
	}
}

/*
	Insert()
*/
BOOL CReportDatabaseService::Insert(long nTot,long nTotal,LPCSTR lpcszData)
{
	BOOL bInserted = FALSE;

	if(LockTable())
	{
		float nRate = ((float)100 * (float)nTot) / (float)nTotal;

		m_ReportTable->ResetMemvars();
		m_ReportTable->PutField_Tot(nTot);
		m_ReportTable->PutField_Rate(nRate);
		m_ReportTable->PutField_Data(lpcszData);
		m_ReportTable->GatherMemvars();
		bInserted = m_ReportTable->Insert();

		UnlockTable();
	}

	return(bInserted);
}

/*
	LockTable()
*/
BOOL CReportDatabaseService::LockTable(void)
{
	return(m_bOpened ? (m_bLockTable ? m_ReportTable->LockTable() : TRUE) : FALSE);
}

/*
	UnlockTable()
*/
void CReportDatabaseService::UnlockTable(void)
{
	if(m_bOpened && m_bLockTable)
		m_ReportTable->UnlockTable();
}
