/*
	CTable.cpp
	Classe base da cui derivare le tabelle per l'accesso al database.
	Non deriva da CBase ma incorpora l'oggetto per gestire il nome della tabella.
	Include il mutex e le relative funzioni di lock solo per MFC.
	Luca Piergentili, 14/07/99
	lpiergentili@yahoo.com
	http://www.geocities.com/lpiergentili/
*/
#include "env.h"
#include "pragma.h"
#include <stdio.h>
#ifdef _WINDOWS
  #include "window.h"
  #ifdef WIN32_MFC
    #include "CMutexEx.h"
  #endif
#else
  #include "typedef.h"
#endif
#include "CTable.h"

/*
	Lock()
*/
#ifdef WIN32_MFC
BOOL CTable::Lock(unsigned int nTimeout)
{
	DWORD dwRet = ::WaitForSingleObject(m_TableMutex->GetHandle(),(DWORD)nTimeout * 1000L);
	return((dwRet==WAIT_FAILED || dwRet==WAIT_ABANDONED || dwRet==WAIT_TIMEOUT) ? FALSE : TRUE);
}
#endif // WIN32_MFC

/*
	Unlock()
*/
#ifdef WIN32_MFC
BOOL CTable::Unlock(void)
{
	return((BOOL)(m_TableMutex->Unlock()));
}
#endif // WIN32_MFC

/*
	LockTable()
*/
#ifdef WIN32_MFC
BOOL CTable::LockTable(unsigned int nTimeout)
{
	BOOL bLock = FALSE;
	char szBuffer[_MAX_PATH+1] = {0};

	if(!m_TableMutex)
		return(TRUE);

	while(TRUE)
	{
		if(Lock(nTimeout))
		{
			bLock = TRUE;
			break;
		}
		else
		{
			_snprintf(szBuffer,
					sizeof(szBuffer)-1,
					"%s: lock failed, retry again ?",
					m_TableMutex->GetName()
					);
			if(::MessageBox(NULL,szBuffer,"CTable::LockTable()",MB_ICONQUESTION|MB_YESNO|MB_TASKMODAL|MB_SETFOREGROUND|MB_TOPMOST)==IDNO)
				break;
		}
	}

	return(bLock);
}
#endif // WIN32_MFC

/*
	UnlockTable()
*/
#ifdef WIN32_MFC
BOOL CTable::UnlockTable(void)
{
	if(!m_TableMutex)
		return(TRUE);

	return(Unlock());
}
#endif // WIN32_MFC
