/*
	CWildCards.cpp
	Classe base per gestione wildcards (?*).
	Modifiche minori al codice originale di Florian Schintke (vedi sotto).
	Luca Piergentili, 26/06/00
	lpiergentili@yahoo.com
	http://www.geocities.com/lpiergentili/

	Implementation of the UN*X wildcards in C. So they are available in a portable way and can be used
	whereever	needed.
	Author(s):
	Florian Schintke (schintke@cs.tu-berlin.de)
	Dates:
	First editing: unknown, but before 04/02/1997
	Last Change  : 11/10/1998
*/
#include "env.h"
#include "pragma.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "CWildCards.h"

int CWildCards::Match(const char* wildcard,const char* test)
{
	if(strcmp(wildcard,test)==0)
		return(1);

  int fit = 1;
  
  for (; ('\000' != *wildcard) && (1 == fit) && ('\000' != *test); wildcard++)
    {
      switch (*wildcard)
        {
        case '[':
	  wildcard++; /* leave out the opening square bracket */ 
          fit = set (&wildcard, &test);
	  /* we don't need to decrement the wildcard as in case */
	  /* of asterisk because the closing ] is still there */
          break;
        case '?':
          test++;
          break;
        case '*':
          fit = asterisk (&wildcard, &test);
	  /* the asterisk was skipped by asterisk() but the loop will */
	  /* increment by itself. So we have to decrement */
	  wildcard--;
          break;
        default:
          fit = (int) (*wildcard == *test);
          test++;
        }
    }
  while ((*wildcard == '*') && (1 == fit)) 
    /* here the teststring is empty otherwise you cannot */
    /* leave the previous loop */ 
    wildcard++;
  return (int) ((1 == fit) && ('\0' == *test) && ('\0' == *wildcard));
}

int CWildCards::set(const char** wildcard,const char** test)
{
  int fit = 0;
  int negation = 0;
  int at_beginning = 1;

  if ('!' == **wildcard)
    {
      negation = 1;
      (*wildcard)++;
    }
  while ((']' != **wildcard) || (1 == at_beginning))
    {
      if (0 == fit)
        {
          if (('-' == **wildcard) 
              && ((*(*wildcard - 1)) < (*(*wildcard + 1)))
              && (']' != *(*wildcard + 1))
	      && (0 == at_beginning))
            {
              if (((**test) >= (*(*wildcard - 1)))
                  && ((**test) <= (*(*wildcard + 1))))
                {
                  fit = 1;
                  (*wildcard)++;
                }
            }
          else if ((**wildcard) == (**test))
            {
              fit = 1;
            }
        }
      (*wildcard)++;
      at_beginning = 0;
    }
  if (1 == negation)
    /* change from zero to one and vice versa */
    fit = 1 - fit;
  if (1 == fit) 
    (*test)++;

  return (fit);
}

int CWildCards::asterisk(const char** wildcard,const char** test)
{
  /* Warning: uses multiple returns */

  int fit = 1;
  char *oldwildcard, *oldtest;

  /* erase the leading asterisk */
  (*wildcard)++; 
  while (('\000' != (**test))
	 && (('?' == **wildcard) 
	     || ('*' == **wildcard)))
    {
      if ('?' == **wildcard) 
	(*test)++;
      (*wildcard)++;
    }
  /* Now it could be that test is empty and wildcard contains */
  /* aterisks. Then we delete them to get a proper state */
  while ('*' == (**wildcard))
    (*wildcard)++;

  if (('\0' == (**test)) && ('\0' != (**wildcard)))
    return (fit = 0);
  if (('\0' == (**test)) && ('\0' == (**wildcard)))
    return (fit = 1); 
  else
    {
      /* Neither test nor wildcard are empty!          */
      /* the first character of wildcard isn't in [*?] */
      oldwildcard = (char*)*wildcard;
      oldtest = (char*)*test;
      do 
	{
	  if (0 == Match(*wildcard, (*test)))
	    oldtest++;
	  *wildcard = oldwildcard;
	  *test = oldtest;
	  /* skip as much characters as possible in the teststring */
	  /* stop if a character match occurs */
	  while (((**wildcard) != (**test)) 
		 && ('['  != (**wildcard))
		 && ('\0' != (**test)))
	    (*test)++;
	  oldwildcard = (char*)*wildcard;
	  oldtest = (char*)*test;
	}
      while ((('\0' != **test))? 
	     (0 == Match (*wildcard, (*test))) 
	     : (0 != (fit = 0)));
      if (('\0' == **test) && ('\0' == **wildcard))
	fit = 1;
      return (fit);
    }
}
