/*
	CFindFile.h
	Classe base per la ricerca files (SDK).
	Luca Piergentili, 14/02/00
	lpiergentili@yahoo.com
	http://www.geocities.com/lpiergentili/
*/
#ifndef _CFINDFILE_H
#define _CFINDFILE_H 1

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include "window.h"
#include "CNodeList.h"

#define _A_ALLFILES (_A_NORMAL|_A_RDONLY|_A_HIDDEN|_A_SYSTEM)
#define _A_ALLFILESANDDIRECTORIES (_A_NORMAL|_A_RDONLY|_A_HIDDEN|_A_SYSTEM|_A_SUBDIR|_A_ARCH)

struct FINDFILE {
	UINT		attrib;							// attributo
	UINT		wr_time;							// ora nel formato MS-DOS
	UINT		wr_date;							// date nel formato MS-DOS
	DWORD	size;							// dimensione (in bytes)
	char		name[_MAX_PATH+_MAX_FNAME+1];			// nome file (pathname incluso)
	HANDLE	handle;							// handle per findfirst/next
};

class CFindFileList : public CNodeList
{
public:
	CFindFileList() {}
	virtual ~CFindFileList() {CNodeList::DeleteAll();}
	BOOL PreDelete(ITERATOR iter)
	{
		delete ((FINDFILE*)iter->data);
		iter->data = (FINDFILE*)NULL;
		return(TRUE);
	}
	int Size(void) {return(sizeof(FINDFILE));}
#ifdef _DEBUG
	const char* Signature(void) {return("CFindFileList");}
#endif
};

class CFindFile
{
public:
	// ctor/dtor
	CFindFile();
	virtual ~CFindFile();

	// controlla l'esistenza
	BOOL		Exist		(LPCSTR lpcszFileName);			// nome file
	BOOL		ExistEx		(LPCSTR lpcszSkel);				// nome file/skeleton

	// ricerca semplice (while)
	LPCSTR	Find			(LPCSTR lpcszSkel = "\\*.*");		// skeleton

	// ricerca ricorsiva (while)
	LPCSTR	FindEx		(LPCSTR lpcszStartDir = "\\",		// dir iniziale (terminare con '\')
						 LPCSTR lpcszSkel = "*.*",		// skeleton
						 BOOL bRecursive = TRUE,			// ricorsivo
						 UINT uAttribute = _A_ALLFILES);	// attributi

	// ricerca ricorsiva (lista)
	UINT		FindFile		(LPCSTR lpcszStartDir = "\\",		// dir iniziale (terminare con '\')
						 LPCSTR lpcszSkel = "*.*",		// skeleton
						 BOOL bRecursive = TRUE,			// ricorsivo
						 UINT uAttribute = _A_ALLFILES);	// attributo
	LPCSTR	GetAt		(int nIndex);

	// files trovati
	UINT		Count		(void) {return(m_nCount);}

	// data/ora
	BOOL		SetFileTime	(LPCSTR lpcszFileName,UINT uDate,UINT uTime);
	BOOL		GetFileTime	(LPCSTR lpcszFileName,UINT* lpuDate,UINT* lpuTime);
	
	// dimensione
	DWORD	GetFileSize	(LPCSTR lpcszFileName);

	// pathnames
	void		SplitPathname	(LPCSTR lpcszPathName,LPSTR lpszDirectory,UINT uDirSize,LPSTR lpszFileName,UINT uFileSize);
	BOOL		CreatePathname	(LPCSTR lpcszPathName);

private:
	int				m_nCount;
	int				m_nInternalCount;
	BOOL				m_bFindFirstCall;
	BOOL				m_bFindExFirstCall;
	WIN32_FIND_DATA	m_FindData;
	FINDFILE			m_FindFile;
	CFindFileList		m_FileList;

	HANDLE	Search	(LPCSTR lpcszStartDir,
					 LPCSTR lpcszSkel,
					 BOOL bRecursive,
					 UINT uAttribute);
	BOOL		First	(LPCSTR lpcszFileName,
					 UINT uAttribute,
					 FINDFILE* f = NULL);
	BOOL		Next		(FINDFILE* f = NULL);
	BOOL		Close	(HANDLE hHandle = INVALID_HANDLE_VALUE);
};

#endif // _CFINDFILE_H
