/*
	CHttp.h
	Classe base per la struttura per la connessione HTTP.
	Luca Piergentili, 05/10/00
	lpiergentili@yahoo.com
	http://www.geocities.com/lpiergentili/
*/
#ifndef _CHTTP_H
#define _CHTTP_H 1

#include "typedef.h"
#include "window.h"
#include <stdio.h>
#include <stdlib.h>
#include "CNodeList.h"
#include "CSock.h"
#include "CDateTime.h"

/*
	PROXY
	struttura per i dati del proxy
*/
struct PROXY {
	char	address[HOSTNAME_SIZE+1];	// indirizzo, specificare ip o nome host
	int	port;					// porta per connessione
};

/*
	COOKIE
	struttura per i dati del cookie
*/
#define COOKIE_NAME_LEN		_MAX_PATH
#define COOKIE_VALUE_LEN		1024
#define COOKIE_EXPIRES_LEN	MAX_DATE_STRING
#define COOKIE_DOMAIN_LEN	HOSTNAME_SIZE
#define COOKIE_PATH_LEN		_MAX_PATH
#define COOKIE_MAX_LEN		(COOKIE_NAME_LEN+COOKIE_VALUE_LEN+COOKIE_EXPIRES_LEN+COOKIE_DOMAIN_LEN+COOKIE_PATH_LEN)
#define COOKIE_REALLY_MAX_LEN	4096 // Netscape

struct COOKIE {
	char	name[COOKIE_NAME_LEN+1];
	char	value[COOKIE_VALUE_LEN+1];
	char	expires[COOKIE_EXPIRES_LEN+1];
	char	path[COOKIE_PATH_LEN+1];
	char	domain[COOKIE_DOMAIN_LEN+1];
	int	secure;
};

/*
	HTTP_PORT
	porta http di default
*/
#define HTTP_PORT 80

/*
	HTTPOBJECTSTAT
	enum per gli stati dell'oggetto HTTP
*/
enum HTTPOBJECTSTAT {
	HTTPOBJECTSTAT_VOID = 0,				// lasciare a 0 in modo tale che per resettare HTTPOBJECT sia sufficente una memset
	HTTPOBJECTSTAT_BEGIN_RECEIVING = 1,	// IDOK, IDCANCEL
	HTTPOBJECTSTAT_RECEIVING = 2,			// IDOK, IDCANCEL
	HTTPOBJECTSTAT_END_RECEIVING = 3,		// IDOK, IDCANCEL
	HTTPOBJECTSTAT_SET_COOKIE = 4,		// IDYES/IDOK, IDNO, IDCANCEL
	HTTPOBJECTSTAT_GET_COOKIE = 5			// IDYES/IDOK, IDNO, IDCANCEL
};

/*
	HTTPOBJECT
	struttura per l'oggetto HTTP (definisce le proprieta' dell'oggetto della connessione)
*/
struct HTTPOBJECT {
	HTTPOBJECTSTAT stat;				// status
	char		location[_MAX_PATH+1];		// url
	char		filename[_MAX_PATH+1];		// nome del file locale
	char		type[_MAX_PATH+1];			// content-type
	DWORD	amount;					// bytes ricevuti
	DWORD	size;					// dimensione totale
	DWORD	starttime;				// inizio download (in millisecondi)
	DWORD	endtime;					// fine download (in millisecondi)
	COOKIE	cookie;					// cookie
};

/*
	HTTPHEADER
	struttura per il campo dell'header HTTP
*/
#define HTTPHEADER_NAME_LEN	32
#define HTTPHEADER_VALUE_LEN	_MAX_PATH

struct HTTPHEADER {
	char name[HTTPHEADER_NAME_LEN+1];
	char value[HTTPHEADER_VALUE_LEN+1];
};

/*
	CHttpHeaderList
	classe per la lista dei campi dell'header HTTP
*/
class CHttpHeaderList : public CNodeList
{
public:
	CHttpHeaderList() {}
	virtual ~CHttpHeaderList() {CNodeList::DeleteAll();}
	BOOL PreDelete(ITERATOR iter)
	{
		delete ((HTTPHEADER*)iter->data);
		iter->data = (HTTPHEADER*)NULL;
		return(TRUE);
	}
	int Size(void) {return(sizeof(HTTPHEADER));}
#ifdef _DEBUG
	const char* Signature(void) {return("CHttpHeaderList");}
#endif
};

/*
	CHttpConnection
	classe per il collegamento HTTP
*/
class CHttpConnection : public CSock
{
public:
	/*
		HTTP_STATUS
		codici di ritorno HTTP (RFC1945 + utente) - lasciare UNKNOW come ultimo per il calcolo dell'indice
	*/
	enum HTTP_STATUS {
		HTTP_STATUS_VOID = 0,
		HTTP_STATUS_NOERROR			= 304,

		HTTP_STATUS_SUCCESSFUL		= 299,
		// successful 2xx 
		HTTP_STATUS_OK				= 200,
		HTTP_STATUS_CREATED			= 201,
		HTTP_STATUS_ACCEPTED		= 202,
		HTTP_STATUS_NO_CONTENT		= 204,
		HTTP_STATUS_PARTIAL_CONTENTS	= 206,

		HTTP_STATUS_REDIRECTION		= 399,
		// redirection 3xx
		HTTP_STATUS_MULTIPLE_CHOICES	= 300,
		HTTP_STATUS_MOVED_PERMANENTLY	= 301,
		HTTP_STATUS_MOVED_TEMPORARILY	= 302,
		HTTP_STATUS_NOT_MODIFIED		= 304,

		HTTP_STATUS_CLIENT_ERROR		= 499,
		// client error 4xx
		HTTP_STATUS_BAD_REQUEST		= 400,
		HTTP_STATUS_UNAUTHORIZED		= 401,
		HTTP_STATUS_FORBIDDEN		= 403,
		HTTP_STATUS_NOT_FOUND		= 404,

		HTTP_STATUS_SERVER_ERROR		= 599,
		// server error 5xx
		HTTP_STATUS_INTERNAL		= 500,
		HTTP_STATUS_NOT_IMPLEMENTED	= 501,
		HTTP_STATUS_BAD_GATEWAY		= 502,
		HTTP_STATUS_UNAVAILABLE		= 503,

		// interni
		HTTP_STATUS_BELOW_MIN_SIZE	= 900,
		HTTP_STATUS_ABOVE_MAX_SIZE	= 901,
		HTTP_STATUS_UNABLE_TO_CREATE_LOCAL_FILE = 902,
		HTTP_STATUS_UNKNOW			= 1001
	};
	enum HTTPSTATUSLEN {
		HTTP_STATUS_LEN = ((int)HTTP_STATUS::HTTP_STATUS_UNKNOW+1)
	};

	// costruttore/distruttore
	CHttpConnection() {CHttpConnection::Reset();}
	virtual ~CHttpConnection() {}

	//inizializza solo gli interni, i modificabili vengono inizializzati dal costruttore
	void			Reset			(void);

	// chiude la connessione
	BOOL			Close			(void);
	
	// opzioni
	inline BOOL	GetSkipExisting	(void)					{return(m_bSkipExisting);}
	inline void	SetSkipExisting	(BOOL bSkipExisting)		{m_bSkipExisting = bSkipExisting;}
	inline UINT	GetConnectionRetry	(void)					{return(m_nConnectionRetry);}
	inline void	SetConnectionRetry	(UINT nRetry)				{m_nConnectionRetry = nRetry;}
	inline UINT	GetConnectionTimeout(void)					{return(m_nConnectionTimeout);}
	inline void	SetConnectionTimeout(UINT nTimeout)			{m_nConnectionTimeout = nTimeout;}

	// nome del server HTTP
	inline LPCSTR	GetServerName		(void)					{return(m_szServerName);}
	inline void	SetServerName		(LPCSTR lpcszServerName)		{strcpyn(m_szServerName,lpcszServerName,sizeof(m_szServerName));}

	// nome dell'host
	inline LPCSTR	GetHostName		(void)					{return(m_szHostName);}
	inline void	SetHostName		(LPCSTR lpcszHostName)		{strcpyn(m_szHostName,lpcszHostName,sizeof(m_szHostName));}
	
	// indirizzo ip dell'host
	inline LPCSTR	GetHostAddr		(void)					{return(m_szHostAddr);}
	inline void	SetHostAddr		(LPCSTR lpcszHostAddr)		{strcpyn(m_szHostAddr,lpcszHostAddr,sizeof(m_szHostAddr));}

	// proxy
	inline LPCSTR	GetProxyAddress	(void)					{return(m_Proxy.address[0]!='\0' ? m_Proxy.address : NULL);}
	inline void	SetProxyAddress	(LPCSTR lpcszProxyAddress)	{strcpyn(m_Proxy.address,lpcszProxyAddress,HOSTNAME_SIZE+1);}
	inline UINT	GetProxyPortNumber	(void)					{return(m_Proxy.port);}
	inline void	SetProxyPortNumber	(UINT nProxyPortNumber)		{m_Proxy.port = nProxyPortNumber;}

	// cookies
	inline BOOL	CookiesAllowed		(void)					{return(m_bAreCookiesAllowed);}
	inline void	AllowCookies		(BOOL bAllow)				{m_bAreCookiesAllowed = bAllow;}
	inline void	SetCookieFileName	(LPCSTR lpcszCookieFileName)	{strcpyn(m_szCookieFileName,lpcszCookieFileName,sizeof(m_szCookieFileName));}
	inline LPCSTR	GetCookieFileName	(void)					{return(m_szCookieFileName);}
	void			ParseCookie		(LPCSTR lpcszCookie,COOKIE* cookie);
	BOOL			ParseInternalCookie	(LPCSTR lpcszCookie,COOKIE* cookie);
	BOOL			SetCookie			(COOKIE* cookie);
	int			GetCookies		(LPCSTR lpcszHost,LPCSTR lpcszPath);
	BOOL			UpdateCookies		(void);
	BOOL			IsCookieExpired	(LPCSTR lpcszDate);
	HTTPHEADER*	EnumCookies		(void);

	// accesso all'oggetto HTTP (HTTPOBJECT)
	inline HTTPOBJECT*		GetHttpObject			(void) {return(&m_HttpObject);}
	inline HTTPOBJECTSTAT	GetHttpObjectStat		(void) {return(m_HttpObject.stat);}
	inline LPCSTR			GetHttpObjectLocation	(void) {return(m_HttpObject.location);}
	inline LPCSTR			GetHttpObjectFileName	(void) {return(m_HttpObject.filename);}
	inline DWORD			GetHttpObjectAmount		(void) {return(m_HttpObject.amount);}
	inline DWORD			GetHttpObjectSize		(void) {return(m_HttpObject.size);}
	inline DWORD			GetHttpObjectStartTime	(void) {return(m_HttpObject.starttime);}
	inline DWORD			GetHttpObjectElapsedTime	(void) {return(m_HttpObject.endtime);}
	inline float			GetHttpObjectDownloadTime(void) {return(((float)(m_HttpObject.endtime - m_HttpObject.starttime)) / 1000.0f);}
	inline COOKIE*			GetHttpObjectCookie		(void) {return(&(m_HttpObject.cookie));}

	// header HTTP
	HTTPHEADER*			EnumHeaders			(void);
	BOOL					AddHeader				(LPCSTR lpcszName,LPCSTR lpcszValue,BOOL bAllowDuplicates = TRUE);
	void					DeleteHeader			(LPCSTR lpcszName);
	inline CHttpHeaderList*	GetHeaderList			(void) {return(&m_HttpHeaderList);}
	
	// gestione errori
	LPCSTR		GetErrorString		(void);
	inline int	GetHttpError		(void)	{return(m_nHttpCode);}
	inline int	GetSocketError		(void)	{return(m_nSocketCode);}
	inline BOOL	IsSocketError		(void)	{return(m_nSocketCode!=0);}
	inline BOOL	IsHttpError		(void)	{return(m_nHttpCode > (int)HTTP_STATUS_SUCCESSFUL);}
	void			ResetSocketError	(void);
	void			ResetHttpError		(void);

	// i codici relativi al socket vengono impostati ricavando i valori della classe base (CSock)
	// mentre i codici http vengono impostati da chi usa la classe per effettuare la connessione
	// notare che i codici winsock vengono impostati solo se il valore corrente e' a 0, ossia se non
	// e' gia' presente un altro codice d'errore, in modo tale che il chiamante possa recuperare il
	// codice reale (in caso contrario errori successivi o la chiusura del socket cambierebbero il
	// valore corrente)
	void			SetSocketError		(void);
	inline void	SetHttpErrorCode	(int nCode)		{m_nHttpCode = nCode;}
	inline void	SetHttpErrorString	(LPCSTR lpcszDesc)	{strcpyn(m_szHttpCode,lpcszDesc,sizeof(m_szHttpCode));}

	inline void	SetMinSize		(UINT nSize)		{m_nMinSize = nSize;}
	inline void	SetMaxSize		(UINT nSize)		{m_nMaxSize = nSize;}
	inline UINT	GetMinSize		(void)			{return(m_nMinSize);}
	inline UINT	GetMaxSize		(void)			{return(m_nMaxSize);}

private:
	int			m_nSocketCode;
	char			m_szSocketCode[_MAX_PATH+1];
	int			m_nHttpCode;
	char			m_szHttpCode[_MAX_PATH+1];
	HTTPOBJECT	m_HttpObject;
	CHttpHeaderList m_HttpHeaderList;
	char			m_szServerName[_MAX_PATH+1];
	char			m_szHostName[_MAX_PATH+1];
	char			m_szHostAddr[_MAX_PATH+1];
	PROXY		m_Proxy;
	BOOL			m_bSkipExisting;
	UINT			m_nConnectionRetry;
	UINT			m_nConnectionTimeout;
	BOOL			m_bAreCookiesAllowed;
	char			m_szCookieFileName[_MAX_PATH+1];
	UINT			m_nMinSize;
	UINT			m_nMaxSize;
};

#endif // _CHTTP_H
