/*
	CUrl.h
	Classe base per la gestione delle URLs.
	Luca Piergentili, 05/10/00
	lpiergentili@yahoo.com
	http://www.geocities.com/lpiergentili/
*/
#ifndef _CURL_H
#define _CURL_H 1

#include "typedef.h"
#include "window.h"
#include <stdlib.h>
#include "CNodeList.h"
#include "CHttp.h"
#include "CSock.h"

#define CURL_FLAG_ENABLE_JAVASCRIPT			0x00000001
#define CURL_FLAG_EXTRACT_JAVASCRIPT_FROM_HTML	0x00000002
#define CURL_FLAG_EXTRACT_TEXT_FROM_HTML		0x00000004

// file html di default
#define INDEX_HTML "index.html"

// url
#define MAX_URL		_MAX_PATH
#define DEFAULT_URL_EXT	".url"

/*
	HTMLTAG_TYPE
	tipi di tag html riconosciuti
*/
enum HTMLTAG_TYPE {
	UNKNOW_TAG,
	
	BASE_TAG,
	A_TAG,
	LINK_TAG,
	IMG_TAG,
	FRAME_TAG,
	EMBED_TAG,
	BLOCKQUOTE_TAG,
	XMLNS_TAG,

	SRC_TAG,
	HREF_TAG,
	DATA_TAG,
	CLASSID_TAG,
	USEMAP_TAG,
	CODEBASE_TAG,
	ARCHIVE_TAG,
	VALUE_TAG,
	
	BEGINSCRIPT_TAG,
	BEGINSCRIPTLANGUAGE_TAG,
	BEGINSCRIPTSRC_TAG,
	ENDSCRIPT_TAG,
	BEGINNOSCRIPT_TAG,
	ENDNOSCRIPT_TAG,
	
	BEGINDIV_TAG,
	ENDDIV_TAG,
	BEGINSPAN_TAG,
	ENDSPAN_TAG,
	BEGINOBJECT_TAG,
	ENDOBJECT_TAG,
	BEGINAPPLET_TAG,
	ENDAPPLET_TAG
};

/*
	HTMLTAG
	struttura per il tag html
*/
#define MAX_HTML_TAG 16

struct HTMLTAG {
	char			name[MAX_HTML_TAG+1];
	char			specifier[MAX_HTML_TAG+1];
	int			len;
	HTMLTAG_TYPE	type;
	HTMLTAG_TYPE	refer;
};

/*
	URL_TYPE
	tipi di url riconosciuti
*/
enum URL_TYPE {
	UNKNOW_URL,
	CID_URL,
	CLSID_URL,
	FILE_URL,
	FINGER_URL,
	FTP_URL,
	GOPHER_URL,
	HDL_URL,
	HTTP_URL,
	HTTPS_URL,
	ILU_URL,
	IOR_URL,
	IRC_URL,
	JAVA_URL,
	JAVASCRIPT_URL,
	LIFN_URL,
	MAILTO_URL,
	MID_URL,
	NEWS_URL,
	NNTP_URL,
	PATH_URL,
	PROSPERO_URL,
	RLOGIN_URL,
	SERVICE_URL,
	SHTTP_URL,
	SNEWS_URL,
	STANF_URL,
	TELNET_URL,
	TN3270_URL,
	WAIS_URL,
	WHOIS_URL
};

/*
	URLTYPE
	struttura per i tipi di url
*/
struct URLTYPE {
	char url[MAX_URL+1];
	URL_TYPE type;
};

/*
	URL
	struttura per i dati delle url
*/
#define MAX_URL_HOST	HOSTNAME_SIZE
#define MAX_URL_DIR		_MAX_PATH
#define MAX_URL_FILE	_MAX_FNAME+_MAX_EXT
#define MAX_URL_DELIM	2
#define MAX_URL_CGI		_MAX_FNAME+_MAX_EXT
#define MAX_URL_QUERY	_MAX_PATH

struct URL {
	char	host[MAX_URL_HOST+1];
	char	dir[MAX_URL_DIR+1];
	char	file[MAX_URL_FILE+1];
	char	delim[MAX_URL_DELIM+1];
	char	cgi[MAX_URL_CGI+1];
	char	query[MAX_URL_QUERY+1];
	int	port;
};

/*
	HREF
	struttura per la lista delle referenze (links)
*/
struct HREF {
	char	href[MAX_URL+1];
};

/*
	CHRefList()
	classe per la lista delle referenze
*/
class CHRefList : public CNodeList
{
public:
	CHRefList() {}
	virtual ~CHRefList() {CNodeList::DeleteAll();}
	BOOL PreDelete(ITERATOR iter)
	{
		delete ((HREF*)iter->data);
		iter->data = (HREF*)NULL;
		return(TRUE);
	}
	int Size(void) {return(sizeof(HREF));}
#ifdef _DEBUG
	const char* Signature(void) {return("CHRefList");}
#endif
};

/*
	HTMLFILETYPE
	struttura per i tipi di file riconosciuti
*/
struct HTMLFILETYPE {
	char	ext[(_MAX_EXT*2)+1];
	int	len;
};

/*
	CHtmlFileTypeList
	classe per la lista dei tipi di file riconosciuti
*/
class CHtmlFileTypeList : public CNodeList
{
public:
	CHtmlFileTypeList() {}
	virtual ~CHtmlFileTypeList() {CNodeList::DeleteAll();}
	BOOL PreDelete(ITERATOR iter)
	{
		delete ((HTMLFILETYPE*)iter->data);
		iter->data = (HTMLFILETYPE*)NULL;
		return(TRUE);
	}
	int Size(void) {return(sizeof(HTMLFILETYPE));}
#ifdef _DEBUG
	const char* Signature(void) {return("CHtmlFileTypeList");}
#endif
};

/*
	URLFILETYPE
	struttura per i tipi di file da riconoscere
*/
struct URLFILETYPE {
	char	ext[(_MAX_EXT*2)+1];
	int	len;
};

/*
	CUrlFileTypeList
	classe per la lista dei tipi di file da riconoscere
*/
class CUrlFileTypeList : public CNodeList
{
public:
	CUrlFileTypeList() {}
	virtual ~CUrlFileTypeList() {CNodeList::DeleteAll();}
	BOOL PreDelete(ITERATOR iter)
	{
		delete ((HTMLFILETYPE*)iter->data);
		iter->data = (HTMLFILETYPE*)NULL;
		return(TRUE);
	}
	int Size(void) {return(sizeof(HTMLFILETYPE));}
#ifdef _DEBUG
	const char* Signature(void) {return("CUrlFileTypeList");}
#endif
};

/*
	HTMLOBJECTSTAT
	enum per gli stati del parsing
*/
enum HTMLOBJECTSTAT {
	HTMLOBJECTSTAT_VOID			= 0, // lasciare a indice 0 in modo tale che per resettare sia sufficente una memset
	HTMLOBJECTSTAT_BEGINPARSING	= 1,
	HTMLOBJECT_PARSING			= 2,
	HTMLOBJECT_ENDPARSING		= 3
};

/*
	HTMLOBJECT
	struttura per la callback per i dati relativi all'analisi del file
*/
struct HTMLOBJECT {
	HTMLOBJECTSTAT stat;			// status (inizio, parsing, fine)
	char		filename[_MAX_PATH+1];	// file
	DWORD	amount;				// bytes analizzati
	DWORD	size;				// dimensione totale
};

/*
	CUrl
	classe per la gestione delle url
*/
class CUrl
{
public:
	CUrl();
	virtual ~CUrl() {}

	// tipo url
	BOOL		IsUrl				(LPCSTR lpcszUrl);
	BOOL		IsUrlType				(LPCSTR lpcszUrl,URL_TYPE url_type);
	URL_TYPE	GetUrlType			(LPCSTR lpcszUrl);

	BOOL		IsCgiUrl				(LPCSTR lpcszUrl);
	LPCSTR	GetCgiUrl				(LPCSTR lpcszUrl);
	LPSTR	GetCgiScript			(LPCSTR lpcszUrl,LPSTR lpszCgiScript,UINT nCgiScriptSize);
	LPSTR	GetCgiQuery			(LPCSTR lpcszUrl,LPSTR lpszCgiQuery,UINT nCgiQuerySize);

	BOOL		AddHtmlFile			(LPCSTR lpcszExt);
	BOOL		IsHtmlFile			(LPCSTR lpcszFileName);
	BOOL		AddUrlFile			(LPCSTR lpcszExt);
	BOOL		IsUrlFile				(LPCSTR lpcszFileName);
	BOOL		IsParentHttpUrl		(LPCSTR lpcszParentUrl,LPCSTR lpcszUrl);
	BOOL		CompareHttpHost		(LPCSTR lpcszHost,LPCSTR lpcszUrl);
	
	// codifica/decodifica
	LPSTR	EncodeToHTML			(LPSTR s,int n);
	LPSTR	EncodeUrl				(LPSTR lpszUrl,UINT nSize);
	LPSTR	DecodeUrl				(LPSTR lpszUrl);
	LPSTR	EncodeHex				(LPSTR s,UINT n,LPCSTR exclude = "",LPCSTR include = "");
	char		DecodeHex				(LPSTR s);

	// parsing
	LPCSTR	StripUrlType			(LPCSTR lpcszUrl);
	LPCSTR	StripPathFromUrl		(LPCSTR lpcszUrl);
	LPCSTR	StripPathFromFile		(LPCSTR lpcszFileName);
	LPSTR	StripParentFromHttpUrl	(LPSTR lpszUrl,UINT nUrlSize);
	BOOL		SplitHttpUrl			(LPCSTR lpcszUrl,URL* pUrl);
	CHRefList* ExtractLinks			(LPCSTR lpcszFile,LPCSTR lpcszBaseUrl,DWORD dwFlags,FPCALLBACK lpfnCallback = NULL,LPVOID lpVoid = NULL);
	
	// pathnames
	BOOL		CreatePathnameFromHttpUrl(LPCSTR lpcszUrl,LPSTR lpszPathname,UINT nPathname,LPCSTR lpcszInitialDir = NULL);
	LPSTR	LocalFileToUrl			(LPCSTR lpcszFile,LPSTR lpszUrl,UINT nUrlSize);
	LPSTR	EnsureValidFilename		(LPSTR lpszFile,BOOL bConvertSlasch = TRUE,LPCSTR lpcszExt = NULL);
	LPSTR	EnsureBackslash		(LPSTR lpszFile,UINT nFileSize);

	// scarica l'url in locale
	UINT		GetHttpUrl			(CHttpConnection*	connection,
								LPCSTR			lpcszUrl,
								LPSTR			lpszTranslatedUrl,
								LPCSTR			lpcszDownloadPath,
								LPSTR			lpszFileName,
								UINT				nFileNameSize,
								BOOL				bForceFileName = FALSE,
								FPCALLBACK		lpfnCallback = NULL,
								LPVOID			lpVoid = NULL);
	
	void		SetHttpPort			(UINT nPort)	{m_nHttpPort = nPort;}
	UINT		GetHttpPort			(void)		{return(m_nHttpPort);}
	UINT		GetDownloadBufferSize	(void)		{return(MAX_DOWNLOAD_BUFFER_SIZE);}

private:
	BOOL		ExtractLinksFromHtmlTag	(LPCSTR lpcszHtmlTag,HTMLTAG_TYPE& htmltag,LPSTR lpszLink,int nLinkSize);
	UINT		ExtractLinksFromBuffer	(LPSTR szBuffer,URL* BaseUrl,URL* Url,DWORD dwFlags);
	BOOL		AddLink				(URL* pBaseUrl,URL* pUrl,LPCSTR szLink,DWORD dwFlags);

	enum {MAX_DOWNLOAD_BUFFER_SIZE = 8192};
	UINT				m_nHttpPort;
	CHRefList			m_HRefList;
	CHtmlFileTypeList	m_HtmlFileTypeList;
	CUrlFileTypeList	m_UrlFileTypeList;
};

#endif // _CURL_H
