/*
	traceexpr.c
	Luca Piergentili, 16/01/97
	lpiergentili@yahoo.com

	Controllo esecuzione, definisce il codice solo se in modalita' _DEBUG.
*/
#define _TRACE_C 1

#if defined(_DEBUG) && defined(_WINDOWS)

#include "env.h"
#include "pragma.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <signal.h>
#define STRICT 1
#include <windows.h>
#include "traceexpr.h"

/* globali */
static int nDeep = 0;
#ifdef TRACE_FILE
	HANDLE hFileHandle;
	HANDLE hFileCallHandle;
#endif
#ifdef TRACE_CONSOLE
	static HANDLE hConsoleHandle = INVALID_HANDLE_VALUE;
#endif

/*
	trace()

	Debug dei dati.
	Notare che la dimensione della stringa di debug non puo' eccedere i TRACE_BUF caratteri.

	input:
	int				flag per trace
	LPCSTR			nome file sorgente
	UINT				numero linea
	LPSTR			formato
	...				parametri per il formato

	output:
	void
*/
void trace(int flag,const char* file,unsigned int line,char* fmt,...)
{
#if defined(TRACE_FILE) || defined(TRACE_CONSOLE)
{
	char* pArgs;
	DWORD dwWrite;
	char trace_buf[TRACE_BUF];
	char buf[(TRACE_BUF/2)];

	if(flag & _TRFLAG_TRACE)
	{
		/* ricava gli argomenti */
		pArgs = (char*)&fmt + sizeof(fmt);
		memset(buf,'\0',sizeof(buf));
		_vsnprintf(buf,sizeof(buf)-1,fmt,pArgs);

		#ifdef TRACE_FILE
		{
			/* costruisce la linea di debug */

			// per indentare la linea con le chiamate di funzione se si mette tutto in un unico file
			//memset(trace_buf,' ',nDeep+2);
			//wsprintf(trace_buf+nDeep+2,"* %s(%d): %s\r\n",file,line,buf);

			memset(trace_buf,'\0',sizeof(trace_buf));
			if(file!=__NOFILE__ && line!=__NOLINE__)
				_snprintf(trace_buf,sizeof(trace_buf)-1,"%s(%d): %s\r\n",file,line,buf);
			else
				_snprintf(trace_buf,sizeof(trace_buf)-1,"%s\r\n",buf);

			/* registra su file */
			if((hFileHandle = CreateFile(TRACE_LOG_FILE,GENERIC_WRITE,0,NULL,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL))!=INVALID_HANDLE_VALUE)
			{
				SetFilePointer(hFileHandle,0L,NULL,FILE_END);
				WriteFile(hFileHandle,trace_buf,strlen(trace_buf),&dwWrite,NULL);
				CloseHandle(hFileHandle);
			}
		}
		#endif

		#ifdef TRACE_CONSOLE
		{
			/* costruisce la linea di debug */
			memset(trace_buf,'\0',sizeof(trace_buf));
			if(file!=__NOFILE__ && line!=__NOLINE__)
				_snprintf(trace_buf,sizeof(trace_buf)-1,"%s(%d): %s\r\n",file,line,buf);
			else
				_snprintf(trace_buf,sizeof(trace_buf)-1,"%s\r\n",buf);

			/* visualizza in finestra */
			if(hConsoleHandle==INVALID_HANDLE_VALUE)
			{
				AllocConsole();
				SetConsoleTitle("TRACE");
				hConsoleHandle = GetStdHandle(STD_OUTPUT_HANDLE);
			}
			if(hConsoleHandle!=INVALID_HANDLE_VALUE)
				WriteConsole(hConsoleHandle,trace_buf,strlen(trace_buf),&dwWrite,NULL);
		}
		#endif
	}
}
#endif /* defined(TRACE_FILE) || defined(TRACE_CONSOLE) */
}

/*
	trace_call()

	Elenco delle chiamate di funzione.

	input:
	int				flag per trace
	int				flag per chiamata (0=entrata, 1=uscita)
	char *			nome funzione

	output:
	void
*/
void trace_call(int flag,int call,char *func)
{
#if defined(TRACE_FILE) || defined(TRACE_CONSOLE)
{
	DWORD dwWrite;
	char trace_buf[TRACE_BUF],buf[(TRACE_BUF/2)];

	if(flag & _TRFLAG_TRACE)
	{
		if(call==0)
		{
			nDeep+=2;
			_snprintf(buf,sizeof(buf)-1,"%c%ds%cs %c%c",'%',nDeep,'%','\r','\n');
			_snprintf(trace_buf,sizeof(trace_buf)-1,buf," ",func);
		}
		else
		{
			_snprintf(buf,sizeof(buf)-1,"%c%ds%cs %c%c",'%',nDeep,'%','\r','\n');
			_snprintf(trace_buf,sizeof(trace_buf)-1,buf," ",func);
			nDeep-=2;
		}

		if(nDeep <= 0)
			lstrcat(trace_buf,"\r\n");

		#ifdef TRACE_FILE
		{
			/* registra su file */
			if((hFileCallHandle=CreateFile(TRACECALL_LOG_FILE,GENERIC_WRITE,0,NULL,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL))!=INVALID_HANDLE_VALUE)
			{
				SetFilePointer(hFileCallHandle,0L,NULL,FILE_END);
				WriteFile(hFileCallHandle,trace_buf,lstrlen(trace_buf),&dwWrite,NULL);
				CloseHandle(hFileCallHandle);
			}
		}
		#endif

		#ifdef TRACE_CONSOLE
		{
			/* visualizza in finestra */
			if(hConsoleHandle==INVALID_HANDLE_VALUE)
			{
				AllocConsole();
				SetConsoleTitle("TRACE");
				hConsoleHandle=GetStdHandle(STD_OUTPUT_HANDLE);
			}
			if(hConsoleHandle!=INVALID_HANDLE_VALUE)
				WriteConsole(hConsoleHandle,trace_buf,lstrlen(trace_buf),&dwWrite,NULL);
		}
		#endif
	}
}
#endif /* defined(TRACE_FILE) || defined(TRACE_CONSOLE) */
}

#ifdef TRACE_FILE
void trace_init(void)
{
	if((hFileHandle=CreateFile(TRACE_LOG_FILE,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL))!=INVALID_HANDLE_VALUE)
		CloseHandle(hFileHandle);
	if((hFileCallHandle = CreateFile(TRACECALL_LOG_FILE,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL))!=INVALID_HANDLE_VALUE)
		CloseHandle(hFileCallHandle);
}
#endif

void assert_expr(void *expr,void *file,unsigned line)
{
	int nCode;
	char assertbuf[TRACE_BUF];
	char assertfmt[] = {
		"Assertion failed:\r\n\r\n"\
		"program: %s\r\n"\
		"file: %s\r\n"\
		"line: %ld\r\n\r\n"\
		"expression:\r\n"\
		"%s\r\n\r\n"\
		"Press Cancel to abort application or Ok to return..."
	};

	char progname[_MAX_PATH+1];
	if(!GetModuleFileName(NULL,progname,sizeof(progname)))
		strcpy(progname,"<unknown>");

	memset(assertbuf,'\0',sizeof(assertbuf));
	_snprintf(assertbuf,sizeof(assertbuf)-1,assertfmt,progname,file,line,expr);

	nCode = MessageBox(NULL,assertbuf,"assert()",MB_OKCANCEL|MB_ICONHAND|MB_TASKMODAL|MB_SETFOREGROUND|MB_TOPMOST);

	if(nCode==IDCANCEL)
	{
		/* raise abort signal */
		raise(SIGABRT);

		/* we usually won't get here, but it's possible that SIGABRT was ignored, so exit the program anyway */
		_exit(3);
	}
	/* Ignore: continue execution */
	else if(nCode==IDOK)
		return;

	abort();
}

#endif /* defined(_DEBUG) && defined(_WINDOWS) */

#undef _TRACE_C
